/*----------------------------------------------------------------------------
 Copyright:      W.Wallucks
 Remarks:        
 known Problems: none
 Version:        04.05.2008
 Description:    messen und regeln mit dem Webmodul

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_OW

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "1-wire/messung.h"
#include "1-wire/ds18x20.h"
#include "1-wire/onewire.h"

#include <util/delay.h>

#include "networkcard/enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "httpd.h"

#include "usart.h"

//#define MES_DEBUG	usart_write 
#define MES_DEBUG(...)	

/**
 * \file
 * Messung.c
 *
 * \author W.Wallucks
 */


#define SENS_INT_ENABLE() 	PCICR |= (1<<SENS_INTENA)
#define SENS_INT_DISABLE() 	PCICR &= ~(1<<SENS_INTENA)

/**
 *	Initialisierung
 */
//void messung_init(void)
//{
//	set_SensorResolution();
//}

/**
 *	neue Temperaturmessung starten
 *	\returns bentigte Wartezeit
 */
uint8_t start_OW(void)
{
#if OW_EXT_POWER
	// T messen: alle gleichzeitig starten
	if ( DS18X20_start_meas( DS18X20_POWER_EXTERN, 0 ) == DS18X20_OK ) {
		// warten bis Messung fertig ist.
		// DS18S20 braucht die vollen 750ms
		// - daher vorsichtshalber volle 750ms warten, falls unterschiedlich bestckt
		return (DS18B20_12_BIT/TIMERBASE);	// Wartezeit zurckgeben
	}

	MES_DEBUG("\r\n*** Messung fehlgeschlagen. (Kurzschluss?) ***");
	return 0;	// es wird kein Timer eingeschaltet
#else
	return 1;	// Timer auf Minimalzeit einschalten
#endif
}

/**
 *	\ingroup messen
 *	bekannte DS18x20 Temperatursensoren auslesen
 */
void lese_Temperatur(void)
{
	uint8_t i = 0;
	uint16_t TWert;
	uint8_t subzero, cel, cel_frac_bits;
	uint8_t tempID[OW_ROMCODE_SIZE];
	
	// erste ID ins RAM holen
	eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE), sizeof(tempID));

#if OW_EXT_POWER
	while (tempID[0] != 0) {
		if ( DS18X20_read_meas( tempID, &subzero,&cel, &cel_frac_bits) == DS18X20_OK ) {
			TWert = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
			if (subzero)
				TWert *= (-1);
			ow_array[i] = TWert;
		}
		else {
			MES_DEBUG("\r\nCRC Error (lost connection?) ");
			DS18X20_show_id_uart( tempID, OW_ROMCODE_SIZE );
		}

		// nchste ID ins RAM holen
		eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE+(OW_ROMCODE_SIZE*++i)),sizeof(tempID));
	}
#else
	while (tempID[0] != 0) {
		// T messen (einzeln starten und messen bei parasitrer Spannungsversorgung)
		if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, tempID ) == DS18X20_OK ) {

			// warten bis Messung fertig ist. DS18S20 braucht die vollen 750ms
			if (tempID[0] == DS18B20_ID)
				_delay_ms(DS18B20_TCONV_10BIT);
			else
				_delay_ms(DS18S20_TCONV);

			if ( DS18X20_read_meas( tempID, &subzero,&cel, &cel_frac_bits) == DS18X20_OK ) {
				TWert = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
				if (subzero)
					TWert *= (-1);
				ow_array[i] = TWert;
			}
			else {
				MES_DEBUG("\r\nCRC Error (lost connection?) ");
				DS18X20_show_id_uart( tempID, OW_ROMCODE_SIZE );
			}
		}
		else MES_DEBUG("\r\n*** Messung fehlgeschlagen. (Kurzschluss?) ***");

		// nchste ID ins RAM holen
		eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE+(OW_ROMCODE_SIZE*++i)),sizeof(tempID));
	}
#endif
}

/**
 *  DS18B20 auf 10bit Genauigkeit setzen (das reicht im Normalfall!)
 *  das verkrzt die Messzeit pro Sensor (750 msec bei 12 bit) auf ein Viertel!
 */
//void set_SensorResolution(void)
void messung_init(void)
{
	uint8_t i = 0;
	uint8_t tempID[OW_ROMCODE_SIZE];

	// erste ID ins RAM holen
	eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE), sizeof(tempID));

	while (tempID[0] == DS18B20_ID) {
		// set 10-bit Resolution - Alarm-low-T 0 - Alarm-high-T 85
		DS18X20_write_scratchpad( tempID , 0, 85, DS18B20_10_BIT);
		// nchste ID ins RAM holen
		eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE+(OW_ROMCODE_SIZE*++i)),sizeof(tempID));
	}
}
#endif
