/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        12.01.2010
 Description:    Commando Interpreter

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "config.h"
#include <string.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "cmd.h"
#include "networkcard/enc28j60.h"
#include "usart.h"
#include "stack.h"
#include "timer.h"

#if USE_HTTPD
	#include "httpd.h"
#endif

#if USE_NTP
	#include "ntp.h"
#endif

#if USE_WOL
	#include "wol.h"
#endif

#if USE_DNS
	#include "dnsc.h"
#endif

#if USE_MAIL
	#include "sendmail.h"
#endif

#if USE_OW
	#include "1-wire/onewire.h"
	#include "1-wire/ds18x20.h"
	#include "1-wire/messung.h"
#endif

#if USE_SYSLOG
	#include "syslog/syslog.h"
#endif

#if USE_TFTPBOOT
	#include <avr/wdt.h>
#endif

#if TFTP_CMD
	volatile unsigned char tftp_server_ip[4];
#endif

#if SDCARD_COMMANDS
	#include "sdcard/fat16.h"
	#include "sdcard/sd_raw.h"
	#include "sdcard/sdcard.h"
	#include "sdcard/partition.h"
#endif

#if TWI_24Cxx
	#include "i2c/i2cmaster.h"
	#include "i2c/24Cxx.h"
#endif

#if TWI_SERVO
	#include "i2c/i2cmaster.h"
	#include "i2c/i2c-servo.h"
#endif

void long_delay(uint16_t ms) {
    for(; ms>0; ms--) _delay_ms(1);
}

char * wortvar[MAX_VAR];
char ret;

volatile unsigned int variable[MAX_VAR];

COMMAND_STRUCTUR COMMAND_TABELLE[] = // Befehls-Tabelle
{
	#if SDCARD_COMMANDS
	{"CAT",command_catfile},		// Cmd mit Textvariable - mu ganz oben stehen
	{"DEL",command_delfile},		// Cmd mit Textvariable - mu ganz oben stehen
	#endif
	#if TFTP_CMD
	{"TFTPFILE",command_tftpfile},	// Cmd mit Textvariable - mu ganz oben stehen
	#endif
	
	#if USE_TFTPBOOT
	{"REBOOT",command_reboot},
	#endif
	
	{"TIME",command_time},
	{"RESET",command_reset},
	{"PORTA",command_porta},
	{"PORTC",command_portc},
	{"PORTCSET",command_setPORTC},
	{"PORTCTOUCH",command_touchPORTC},
	{"IP",command_ip},
	{"NET",command_net},
	{"ROUTER",command_router},
	
	#if USE_SYSLOG
	{"SYSLOG",command_syslog},
	#endif //USE_SYSLOG
	
	#if TFTP_CMD
	{"TFTP",command_tftp},
	#endif
	
	#if USE_NTP
	{"NTP",command_ntp},
	{"NTPR",command_ntp_refresh},	
	#endif
	
	{"MAC",command_mac},
	{"RV",command_readvar},
	{"SV",command_setvar},
	
	#if USE_WOL
	{"WOL",command_wol},
	#endif //USE_WOL
	
	#if USE_NETCMDS
	{"ARP",command_arp},
	{"TCP",command_tcp},
	{"PING", command_ping},
	{"VER",command_ver},
	#endif //USE_NETCMDS
	
	#if USE_OW
	{"OW", command_ow},
	{"OWSCAN", command_owscan},
	{"OWLIST", command_owlist},
	{"OWSET", command_owset},
	#endif //USE_OW
	
	#if USE_EEPVAR
	{"RVE",command_rve},
	{"SVE",command_sve},
	#endif

	#if USE_EEPVAR_CMD
	{"EEP",command_eep},
	{"EEPW",command_eepw},
	#endif
	
	#if SDCARD_COMMANDS
	{"SDINFO",command_diskinfo},
	{"DIR",command_dir},
	#endif

	#if USE_MAIL
    {"mail", command_mail},
    #endif //USE_MAIL
	
	#if TWI_24Cxx
	{"I2CWE", command_writeSL24Cxx},
	{"I2CRE", command_readSL24Cxx},
	#endif
	
	#if TWI_SERVO
	{"SERVO", command_i2cservo},
	#endif
	
	#if HELPTEXT
	{"HELP",command_help},
	{"?",command_help},
	#endif //HELPTEXT
	
	{NULL,NULL} 
};

#if HELPTEXT
	PROGMEM char helptext[] = {
		"RESET  - reset the AVR - Controller\r\n"
		#if USE_TFTPBOOT
		"REBOOT - reboot from TFTP-Server\r\n"
		#endif
		#if USERDEFINED_MAC
		"MAC    - list/change MAC-address 1 0x.. 0x.. 0x.. 0x.. 0x.. 0x..\r\n"
		#else
		"MAC    - list MAC-address\r\n"
		#endif
		"IP     - list/change ip\r\n"
		"NET    - list/change netmask\r\n"
		"ROUTER - list/change router ip\r\n"
		#if USE_NTP
		"NTP    - list/change NTP\r\n"
		"NTPR   - NTP Refresh\r\n"
		"TIME   - get date & time\r\n"
		#else
		"TIME   - get time\r\n"
		#endif
		#if USE_SYSLOG
		"SYSLOG - list/change SYSLOG\r\n"
		#endif //USE_SYSLOG
		#if TFTP_CMD
		"TFTP   - list/change TFTP-Server\r\n"
		"TFTPFILE    - list/change Bootfile\r\n"
		#endif //TFTP_CMD
		#if USE_NETCMDS
        "PING   - send Ping\r\n"
		"ARP    - list the ARP table\r\n"
		"TCP    - list the tcp table\r\n"
		"VER    - list enc version number\r\n"
		#endif
		"PORTA       - list PORTA\r\n"
		"PORTC       - list PORTC\r\n"
		"PORTCSET    - set PORTC 0-7 0/1\r\n"
		"PORTCTOUCH  - toggle PORTC 0-7 for X milliseconds\r\n"
		#if USE_OW
		"OW     - list 1-Wire temperatures\r\n"
		"OWLIST - list 1-Wire sensors\r\n"
		"OWSCAN - detect 1-Wire sensors\r\n"
		"OWSET  - write 1-wire sensor address 0-7 0x.. 0x.. 0x.. 0x.. 0x.. 0x.. 0x.. 0x..\r\n"
		#endif //USE_OW
		"SV     - set variable\r\n"
		"RV     - read variable <val>\r\n"
		#if USE_EEPVAR
		"SVE    - write variables to EEPROM\r\n"
		"RVE    - read variables from EEPROM\r\n"
		#endif

		#if USE_EEPVAR_CMD
		"EEP    - write byte to EEPROM position X BYTE\r\n"
		"EEPW   - write word to EEPROM position X WORD\r\n"
		#endif
		
		#if SDCARD_COMMANDS
		"SDINFO - list SD-Card Information\r\n"
		"CAT    - show filecontent\r\n"
		"DIR    - show directory\r\n"
		"DEL    - delete file\r\n"
		#endif
		
		#if TWI_24Cxx
		"I2CWE  - Write to i2c EEPROM 0-2047 0x..\r\n"
		"I2CRE  - Read from i2c EEPROM 0-2047\r\n"
		#endif
		
		#if TWI_SERVO
		"SERVO  - show / set position <servo> <1000-2000>\r\n"
		#endif

        #if USE_MAIL
        "MAIL   - send E_MAIL\r\n"
        #endif //USE_MAIL
		
		#if USE_WOL
		"WOL    - send WOL / set MAC / set MAC and IP\r\n"
		#endif //USE_WOL

		"HELP   - print Helptext\r\n"
		"?      - print Helptext\r\n"
	};
#endif


//------------------------------------------------------------------------------
//Commando auswerten
unsigned char extract_cmd (char *string_pointer)
{
	//Stringzeiger;
	char *string_pointer_tmp;
	unsigned char cmd_index = 0;
 
    string_pointer_tmp = strsep(&string_pointer," "); 

	//Kommando in Tabelle suchen
	while(strcasecmp(COMMAND_TABELLE[cmd_index].cmd,string_pointer_tmp))
    {
        //Abruch Whileschleife und Unterprogramm verlassen 
        if (COMMAND_TABELLE[++cmd_index].cmd == 0) return(0);
    }
    
	//Variablen finden und auswerten
	for (unsigned char a = 0; a<MAX_VAR;a++)
	{
		#if USE_TEXTVAR
		if(cmd_index < 3) // Befehle 0-2 sind Kommandos mit Textvariable oder ganz ohne
		{
			string_pointer_tmp = strsep(&string_pointer," ");
			wortvar[a] = string_pointer_tmp;
		} else {
		#endif
			string_pointer_tmp = strsep(&string_pointer,":., ");
			variable[a] = strtol(string_pointer_tmp,NULL,0);
		#if USE_TEXTVAR
		}
		#endif
	}

    //Kommando ausfhren
	COMMAND_TABELLE[cmd_index].fp();
	return(1); 
}

//------------------------------------------------------------------------------
//Reset ausfhren
void command_reset (void)
{
	RESET();
}

//------------------------------------------------------------------------------
//print/edit own IP
void command_ip (void)
{
	write_eeprom_ip(IP_EEPROM_STORE);
	(*((unsigned long*)&myip[0])) = get_eeprom_value(IP_EEPROM_STORE,MYIP);
	usart_write("My IP: %1i.%1i.%1i.%1i\r\n",myip[0],myip[1],myip[2],myip[3]);
}

//------------------------------------------------------------------------------
//
void write_eeprom_ip (unsigned int eeprom_adresse)
{
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{	
		//value ins EEPROM schreiben
		for (unsigned char count = 0; count<4;count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(eeprom_adresse + count),variable[count]);
		}
	}
}

//------------------------------------------------------------------------------
#if USE_NTP
//print/edit NTP Server IP
void command_ntp (void)
{
	write_eeprom_ip(NTP_IP_EEPROM_STORE);
	(*((unsigned long*)&ntp_server_ip[0])) = get_eeprom_value(NTP_IP_EEPROM_STORE,NTP_IP);
	usart_write("NTP_Server: %1i.%1i.%1i.%1i\r\n",ntp_server_ip[0],ntp_server_ip[1],ntp_server_ip[2],ntp_server_ip[3]);
}

//Time Refresh via NTP-Server
void command_ntp_refresh (void)
{
	ntp_request();
}
#endif //USE_NTP

//------------------------------------------------------------------------------
//print/edit SYSLOG Server IP
#if USE_SYSLOG
void command_syslog (void)
{
	write_eeprom_ip(SYSLOG_IP_EEPROM_STORE);
	(*((unsigned long*)&syslog_server_ip[0])) = get_eeprom_value(SYSLOG_IP_EEPROM_STORE,SYSLOG_IP);
	usart_write("SYSLOG_Server: %1i.%1i.%1i.%1i\r\n",syslog_server_ip[0],syslog_server_ip[1],syslog_server_ip[2],syslog_server_ip[3]);
}
#endif

//------------------------------------------------------------------------------
//print/edit Netmask
void command_net (void)
{
	write_eeprom_ip(NETMASK_EEPROM_STORE);
	(*((unsigned long*)&netmask[0])) = get_eeprom_value(NETMASK_EEPROM_STORE,NETMASK);
	usart_write("NETMASK: %1i.%1i.%1i.%1i\r\n",netmask[0],netmask[1],netmask[2],netmask[3]);
}

//------------------------------------------------------------------------------
//print/edit Router IP
void command_router (void)
{
	write_eeprom_ip(ROUTER_IP_EEPROM_STORE);
	(*((unsigned long*)&router_ip[0])) = get_eeprom_value(ROUTER_IP_EEPROM_STORE,ROUTER_IP);
	usart_write("Router IP: %1i.%1i.%1i.%1i\r\n",router_ip[0],router_ip[1],router_ip[2],router_ip[3]);
}

//------------------------------------------------------------------------------
//print own mac
void command_mac (void)
{
	#if USERDEFINED_MAC
	// EEPROM beschreiben, falls Parameter angegeben wurden
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{	
		//schreiben der MAC
		for (unsigned char count = 0; count<6; count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(MAC_EEPROM_STORE + count),variable[count+1]);
		}
	} else {
	#endif
	usart_write("MAC: %2x:%2x:%2x:%2x:%2x:%2x\r\n",mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
	#if USERDEFINED_MAC
	}
	#endif
}

#if USE_NETCMDS
//------------------------------------------------------------------------------
//print enc28j60 chip version
void command_ver (void)
{
	usart_write("ENC28J60-Version: %1x\r\n", enc28j60_revision);
}

//------------------------------------------------------------------------------
//print ARP table
void command_arp (void)
{
	for (unsigned char index = 0;index<MAX_ARP_ENTRY;index++)
	{
		usart_write("%2i  MAC:%2x",index,arp_entry[index].arp_t_mac[0]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[1]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[2]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[3]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[4]);
		usart_write(".%2x",arp_entry[index].arp_t_mac[5]);
		
		usart_write("  IP:%3i",(arp_entry[index].arp_t_ip&0x000000FF));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x0000FF00)>>8));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0x00FF0000)>>16));
		usart_write(".%3i",((arp_entry[index].arp_t_ip&0xFF000000)>>24));
			
		usart_write("  Time:%4i\r\n",arp_entry[index].arp_t_time);
	}
}

//------------------------------------------------------------------------------
//print ARP table
void command_tcp (void)
{
	for (unsigned char index = 0;index<MAX_TCP_ENTRY;index++)
	{
		usart_write("%2i",index);
		usart_write("  IP:%3i",(tcp_entry[index].ip&0x000000FF));
		usart_write(".%3i",((tcp_entry[index].ip&0x0000FF00)>>8));
		usart_write(".%3i",((tcp_entry[index].ip&0x00FF0000)>>16));
		usart_write(".%3i",((tcp_entry[index].ip&0xFF000000)>>24));
		usart_write("  PORT:%4i",HTONS(tcp_entry[index].src_port));
		usart_write("  Time:%4i\r\n",tcp_entry[index].time);
	}
}

//------------------------------------------------------------------------------
// Sende "Ping" an angegebene Adresse
void command_ping (void)
{
	if (*((unsigned int*)&variable[0]) != 0x00000000)
	{
		(*((unsigned long*)&ping.ip1[0])) = ((variable[0])+((unsigned long)(variable[1])<<8)+((unsigned long)(variable[2])<<16)+((unsigned long)(variable[3])<<24));
		//ARP Request senden
		arp_request ( (*(unsigned long*)&ping.ip1[0]));
		
		//ICMP-Nachricht Type=8 Code=0: Echo-Anfrage
		//TODO: Sequenznummer, Identifier 
		icmp_send( (*(unsigned long*)&ping.ip1[0]),0x08,0x00,1,1);
	}
}
#endif //USE_NETCMDS

//------------------------------------------------------------------------------
//ndern einer Variable
void command_setvar (void)
{
	var_array[variable[0]] = variable[1];
	usart_write("Inhalt der Variable[%2i] = %2i\r\n",variable[0],var_array[variable[0]]);
}


//------------------------------------------------------------------------------
//lesen einer Variable
void command_readvar (void)
{
	uint8_t i;
	if(variable[0] > 9 )  // kleiner 10 sind Port A,C und D Werte / 1 gibt Remote-Werte aus
	{
		i = variable[0];
		usart_write("Variable %2i = %2i\r\n",i,var_array[i]);
	} else {
		for(i=0;i<MAX_VAR_ARRAY;i++)
		{
		#if USE_UDPSEND
		if(variable[0] == 1)   // Remote Werte ausgeben
		{
			usart_write("Remote Var %2i = %2i\r\n",i,va2_array[i]);
		} else {
			usart_write("Variable %2i = %2i\r\n",i,var_array[i]);
		}
		#else
		usart_write("Variable %2i = %2i\r\n",i,var_array[i]);
		#endif
		}
	}
}

//------------------------------------------------------------------------------
//print Time
void command_time (void)
{
	unsigned char hh = (time/3600)%24;
	unsigned char mm = (time/60)%60;
	unsigned char ss = time %60;
	
	#if USE_NTP
	ntp_struct dt;
	decode_time(time, &dt);
	usart_write ("\n\rDATE: %2i.%2i.%2i - %2i:%2i:%2i\r\n",dt.day,dt.month,dt.year,hh,mm,ss);
	#else
	usart_write ("\n\rTIME: %2i:%2i:%2i\r\n",hh,mm,ss);
	#endif
}

//------------------------------------------------------------------------------
//Sendet eine fertige E-MAIL
#if USE_MAIL
void command_mail (void)
{
	mail_enable = 1;
}
#endif //USE_MAIL

//------------------------------------------------------------------------------
//
#if USE_WOL
void command_wol (void)
{
	// EEPROM beschreiben, falls Parameter angegeben wurden
	if ((*((unsigned int*)&variable[0]) != 0x00000000) || (*((unsigned int*)&variable[1]) != 0x00000000) || (*((unsigned int*)&variable[2]) != 0x00000000))
	{	
		//schreiben der MAC
		for (unsigned char count = 0; count<6; count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(WOL_MAC_EEPROM_STORE + count),variable[count]);
		}
		//zustzlich schreiben der Broadcast-Adresse, falls vorhandenden
		for (unsigned char count = 0; count<4 && (*((unsigned int*)&variable[6]) != 0x00000000);count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char*)(WOL_BCAST_EEPROM_STORE + count),variable[count+6]);
		}
		//init
		wol_init();
	}else{
		//MagicPacket senden
		wol_request();
	}
}
#endif //USE_WOL

//------------------------------------------------------------------------------
//print helptext
void command_help (void)
{
#if HELPTEXT
	unsigned char data;
	PGM_P helptest_pointer = helptext;
	
	do
	{
		data = pgm_read_byte(helptest_pointer++);
		usart_write("%c",data);
	}while(data != 0);
#endif
}

//------------------------------------------------------------------------------
//save all ip-datas
void save_ip_addresses(void)
{
  for (unsigned char count = 0; count<4; count++)
  {
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(IP_EEPROM_STORE + count),myip[count]);
  }
  for (unsigned char count = 0; count<4; count++)
  {
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(NETMASK_EEPROM_STORE + count),netmask[count]);
  }
  for (unsigned char count = 0; count<4; count++)
  {
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(ROUTER_IP_EEPROM_STORE + count),router_ip[count]);
  }
  #if USE_DNS
  for (unsigned char count = 0; count<4; count++)
  {
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(DNS_IP_EEPROM_STORE + count),dns_server_ip[count]);
  }
  #endif //USE_DNS
  #if USE_NTP
  for (unsigned char count = 0; count<4; count++)
  {
    eeprom_busy_wait ();
    eeprom_write_byte((unsigned char *)(NTP_IP_EEPROM_STORE + count),ntp_server_ip[count]);
  }
  #endif //USE_NTP
  #if USE_SYSLOG
  for (unsigned char count = 0; count<4; count++)
  {
	eeprom_busy_wait ();
	eeprom_write_byte((unsigned char *)(SYSLOG_IP_EEPROM_STORE + count),syslog_server_ip[count]);
  }
  #endif //USE_SYSLOG
}

//------------------------------------------------------------------------------
//Read all IP-Datas
void read_ip_addresses (void)
{
  (*((unsigned long*)&myip[0]))      = get_eeprom_value(IP_EEPROM_STORE,MYIP);
  (*((unsigned long*)&netmask[0]))   = get_eeprom_value(NETMASK_EEPROM_STORE,NETMASK);
  (*((unsigned long*)&router_ip[0])) = get_eeprom_value(ROUTER_IP_EEPROM_STORE,ROUTER_IP);
	
	#if USE_DNS
	//DNS-Server IP aus EEPROM auslesen
	(*((unsigned long*)&dns_server_ip[0])) = get_eeprom_value(DNS_IP_EEPROM_STORE,DNS_IP);
	#endif
	#if USE_NTP
	//NTP-Server IP aus EEPROM auslesen
	(*((unsigned long*)&ntp_server_ip[0])) = get_eeprom_value(NTP_IP_EEPROM_STORE,NTP_IP);
	#endif
	#if USE_SYSLOG
	//SYSLOG-Server IP aus EEPROM auslesen
	(*((unsigned long*)&syslog_server_ip[0])) = get_eeprom_value(SYSLOG_IP_EEPROM_STORE,SYSLOG_IP);
	#endif
}

//------------------------------------------------------------------------------
// print PORTA
void command_porta (void)
{
	// Digitalwerte (0/1) -> PORTA0-3
	usart_write ("A0: %i\r\nA1: %i\r\nA2: %i\r\nA3: %i\r\n",(PINA&0b00000001),(PINA&0b00000010)>>1,(PINA&0b00000100)>>2,(PINA&0b00001000)>>3);
	
	#if !USE_ADC
		usart_write ("A4: %i\r\nA5: %i\r\nA6: %i\r\n",(PINA&0b00010000)>>4,(PINA&0b00100000)>>5,(PINA&0b01000000)>>6);
		#if !USE_OW
			usart_write ("A7: %i\r\n",(PINA&0b10000000)>>7);
		#else
			usart_write ("A7: 1-Wire Bus\r\n");
		#endif
	#else
		// Analogwerte (0-1023) -> PORTA4-6
		uint16_t analog;
		uint8_t maxadc	= 8;

        #if USE_OW
                maxadc = maxadc - 1;
        #endif

        for(uint8_t i=ADC_START;i<maxadc;i++)
		{
			analog = (var_array[i]);
			usart_write ("A%i: %i\r\n",i,analog);
		}
		#if USE_OW
			usart_write ("A7: 1-Wire Bus\r\n");
		#endif
	#endif
}

//------------------------------------------------------------------------------
// print Ausgabewert von PORTC0-7
void command_portc (void)
{
	usart_write ("C0: %i\r\nC1: %i\r\n",(PINC&0b00000001)>>0,(PINC&0b00000010)>>1);
	usart_write ("C2: %i\r\nC3: %i\r\nC4: %i\r\n",(PINC&0b00000100)>>2,(PINC&0b00001000)>>3,(PINC&0b00010000)>>4);
	usart_write ("C5: %i\r\nC6: %i\r\nC7: %i\r\n",(PINC&0b00100000)>>5,(PINC&0b01000000)>>6,(PINC&0b10000000)>>7);
	// usart_write ("PORTC: %i\r\n",(PORTC));
}

//------------------------------------------------------------------------------
//PORTC setzen
void command_setPORTC (void)
{
	usart_write("setportC %1i %1i\r\n",variable[0],variable[1]);
	if (variable[1] == 1)
		PORTC = PORTC |= (1<<variable[0]) ;
		//  setzt Bit  in PORT und setzt damit Pin auf high
	else if (variable[1] == 0)
		PORTC = PORTC &= ~(1<<variable[0]);
		//lscht Bit  in PORT und setzt damit Pin auf low
	else
	usart_write("ERROR PARAMETER  [Pin.Status]\r\n");
}

//------------------------------------------------------------------------------
//PORTC setzen und rcksetzen
void command_touchPORTC (void)
{
	usart_write("portCtouch %1i %i\r\n",variable[0],variable[1]);
	if (variable[1] < 100) variable[1] = 100;
	PORTC = PINC ^ ( 1 << variable[0] );
	long_delay(variable[1]);
	PORTC = PINC ^ ( 1 << variable[0] );
}

//------------------------------------------------------------------------------
#if USE_OW

//------------------------------------------------------------------------------
// print Temperaturen
void command_ow (void)
{
//extern volatile int16_t ow_array[OW_MAXSENSORS];

int16_t TWert;
for(uint8_t i=0;i<OW_MAXSENSORS;i++)
	{
	TWert = ow_array[i];
	// Vorzeichen
	if ( ow_array[i] < 0 ) {
		usart_write ("-");
		TWert *= (-1);
	}
	usart_write ("Sensor %i: %i,%i\r\n",i,(TWert/10),(TWert%10));
	}
}

//------------------------------------------------------------------------------
// Zeige Sensorenliste aus EEPROM
void command_owlist (void)
{
for(uint8_t i=0;i<OW_MAXSENSORS;i++)
	{
	//OW ID Ausgeben
	uint8_t tempID[OW_ROMCODE_SIZE];
	eeprom_read_block(tempID,(unsigned char *)(OW_EEPROM_STORE+(OW_ROMCODE_SIZE*i)),sizeof(tempID));
	usart_write ("Sensor %i ID: 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x \r\n",i,tempID[0],tempID[1],tempID[2],tempID[3],tempID[4],tempID[5],tempID[6],tempID[7]);
	// uint8_t sensor[i][OW_ROMCODE_SIZE];
	// usart_write ("Sarray %i ID: 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x 0x%2x \r\n",i,sensor[i][0],sensor[i][1],sensor[i][2],sensor[i][3],sensor[i][4],sensor[i][5],sensor[i][6],sensor[i][7]);
	}
}

//------------------------------------------------------------------------------
// Schreibe Sensor-ID in EEPROM
void command_owset (void)
{
	// EEPROM beschreiben
	if (*((unsigned int*)&variable[0]) < OW_MAXSENSORS)
	{
		//schreiben der MAC
		for (unsigned char count = 0; count<OW_ROMCODE_SIZE; count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char *)(OW_EEPROM_STORE + (OW_ROMCODE_SIZE*variable[0]) + count),variable[count+1]);
		}
	}
}

//------------------------------------------------------------------------------
// Scannen und in EEPROM schreiben des 1-Wire Bus 
void command_owscan(void)
{
	uint8_t id[OW_ROMCODE_SIZE], diff, nSensors, scanid;
	
	// Wenn keine Sensoren im EEPROM
	scanid = eeprom_read_byte((unsigned char *)(OW_EEPROM_STORE));
	if ((scanid == 0x00) || (scanid == 0xFF)) variable[0] = 1;
	
	if (variable[0] == 1)
	{
		usart_write( "Rescanning Bus for DS18X20\r\n" );
		for(uint16_t j=OW_EEPROM_STORE;j<OW_EEPROM_STORE+(OW_ROMCODE_SIZE*OW_MAXSENSORS)+OW_ROMCODE_SIZE;j++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte((unsigned char*)(j),0x00);
		}
	} else {
		usart_write( "Scanning Bus for DS18X20\r\n" );
	}
	
	nSensors = 0;
	uint16_t j = OW_EEPROM_STORE;

	for( diff = OW_SEARCH_FIRST; diff != OW_LAST_DEVICE && nSensors < OW_MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		if( diff == OW_PRESENCE_ERR ) {
			usart_write( "No Sensor found\r\n" );
			break;
		}

		if( diff == OW_DATA_ERR ) {
			usart_write( "Bus Error\r\n" );
			break;
		}

		for (uint16_t i=0;i<OW_ROMCODE_SIZE;i++) 
		{
			usart_write("0x%2x ",id[i]);
			if (variable[0] == 1)
			{
				eeprom_write_byte((unsigned char*)(j),id[i]);
				j++;
			}
		}
		usart_write(" VCC %s\r\n",DS18X20_get_power_status(id)?"extern":"parasite");
		nSensors++;
	}
}
#endif // endif USE_OW

//---------------------------------------------------------------------
// EEPROM Funktionen

#if USE_EEPVAR_CMD
void command_eep (void)
{
        if (*((unsigned int*)&variable[0]) != 0x00000000)
        {
                uint8_t i;
                //Byte aus dem EEPROM lesen
                        i = 0x00;
                        usart_write ("EEPROM Adresse : %i\r\n",variable[0]);
                        i = eeprom_read_byte((unsigned char *)(variable[0]));
                        usart_write ("WERT  :  %i\r\n",i);
                if (*((unsigned int*)&variable[1]) != 0x00000000)
                {
                        usart_write ("EEPROM beschreiben\r\n");
                        eeprom_busy_wait ();
                        eeprom_write_byte((unsigned char*)(variable[0]),variable[1]);
                        i = eeprom_read_byte((unsigned char *)(variable[0]));
                        usart_write ("WERT  :  %i\r\n",i);
                }
        }
}

void command_eepw (void)
{
        if (*((unsigned int*)&variable[0]) != 0x00000000)
        {
                uint16_t i;
                //Byte aus dem EEPROM lesen
                i = 0x00;
                usart_write ("EEPROM Adresse : %i\r\n",variable[0]);
                i = eeprom_read_word((unsigned int *)(variable[0]));
                usart_write ("WERT  :  %i\r\n",i);
                if (*((unsigned int*)&variable[1]) != 0x00000000)
                {
                        usart_write ("EEPROM beschreiben\r\n");
                        eeprom_busy_wait ();
                        eeprom_write_word((unsigned int*)(variable[0]),variable[1]);
                        i = eeprom_read_word((unsigned int*)(variable[0]));
                        usart_write ("WERT  :  %i\r\n",i);
                }
        }
}
#endif //use_EEPVAR_CMD

#if USE_EEPVAR
void command_sve (void)
{
        usart_write ("VAR in EEPROM schreiben\r\n");
        for(uint8_t i=10;i<MAX_VAR_ARRAY;i++)  // ab Arrayposition 10 / < 10 alles Portwerte
        {
                eeprom_busy_wait ();
                eeprom_write_word((unsigned int*)(SV_EEPROM_STORE + (2*(i-10))),var_array[i]);
        }
}

void command_rve (void) // EEPROM Bereich lesen
{
        uint16_t var_temp;
        usart_write ("VAR aus EEPROM lesen\r\n");
        for(uint8_t i=10;i<MAX_VAR_ARRAY;i++)  // ab Arrayposition 10 / < 10 alles Portwerte
        {
                var_temp = eeprom_read_word((unsigned int*)(SV_EEPROM_STORE + (2*(i-10))));
                if (var_temp !=0xFFFF)
                {
                        var_array[i] = var_temp;
                }
        }
}
#endif //use_EEPVAR

//---------------------------------------------------------------------
// EEPROM Grunddaten beim Start auslesen
void eepvar_init (void)
{
		// Ausgnge PORTC werden mit Startwert aus EEPROM geschalten.
        PORTC = eeprom_read_byte((unsigned char *)(DEFPC_EEPROM_STORE));

        #if USE_EEPVAR
        command_rve();	// Flle var_array mit Startwerten aus EEPROM
		#endif

		#if USE_OW
		uint8_t id;
		id = eeprom_read_byte((unsigned char *)(OW_EEPROM_STORE));
		
		if ((id == 0xFF) || (id == 0x00))  {	// Wenn keine Sensoren im EEPROM
			command_owscan();	// Scanne 1-wire Bus nach Sensoren 
		}
		#endif //USE_OW
}

#if USE_TFTPBOOT
//------------------------------------------------------------------------------
// Reboot from TFTP-Bootloader

void command_reboot (void)
{
	cli();		// Disable all interrupts
	// changing interrupt vector to bootloader section only needed if BOOTRST is not used
	char tempval = MCUCR;			// Get MCUCR
	MCUCR = tempval | (1<<IVCE);	// Enable change of Interrupt Vectors
	MCUCR = tempval | (1<<IVSEL);	// Move interrupts to Boot Flash section

	// enable watchdog and release it
	do {
		wdt_enable(WDTO_15MS);
		for(;;){}
	} while(0);
}
#endif

#if TFTP_CMD
//------------------------------------------------------------------------------
// print/edit TFTP-Server IP

void command_tftp (void)
{
	write_eeprom_ip(TFTP_IP_EEPROM_STORE);
	(*((unsigned long*)&tftp_server_ip[0])) = get_eeprom_value(TFTP_IP_EEPROM_STORE,IP(0,0,0,0));
	usart_write("TFTP Server: %1i.%1i.%1i.%1i\r\n",tftp_server_ip[0],tftp_server_ip[1],tftp_server_ip[2],tftp_server_ip[3]);
}

//------------------------------------------------------------------------------
// print/edit TFTP-Filename

void command_tftpfile (void)
{
	char *string2=wortvar[0];
	char c;
	uint16_t j = 0;

	if (strlen(string2) == 0)
	{
		usart_write("Bootfile: ");
		do {
			c = eeprom_read_byte ((unsigned char*)TFTP_FILE_EEPROM_STORE+j);
			usart_write("%c",c);
		} while ((c != 0) && (c != 0xFF) && (j++ < 16) );
		usart_write("\r\n");
	} else {
		if (strlen(string2) < 16)
		{
			for(j=TFTP_FILE_EEPROM_STORE;j<TFTP_FILE_EEPROM_STORE+16;j++)
			{
				eeprom_busy_wait ();
				eeprom_write_byte((unsigned char*)(j),0xFF);
			}
			for(j=0;j<strlen(string2);j++)
			{
				eeprom_busy_wait ();
				eeprom_write_byte((unsigned char*)(TFTP_FILE_EEPROM_STORE + j),string2[j]);
			}
			if ( j > 0)
			{
				eeprom_busy_wait ();
				eeprom_write_byte((unsigned char*)(TFTP_FILE_EEPROM_STORE + j),string2[j]);
			}
		} else {	
			usart_write("ERR: Dateiname zu lang\r\n\r\n");
		}
	}
}

#endif	// TFTP_CMD

//------------------------------------------------------------------------------
// print SC-Card Information

#if SDCARD_COMMANDS
void command_diskinfo(void)
{
	struct sd_raw_info disk_info;
	if(!sd_raw_get_info(&disk_info)) {
		usart_write("Disk kann nicht gelesen werden.");
	} else {
		usart_write("\r\nmanuf:  0x%x",disk_info.manufacturer);
		usart_write("\r\noem:    %s",disk_info.oem);
		usart_write("\r\nprod:   %s",disk_info.product);
		usart_write("\r\nrev:    %i",disk_info.revision);
		usart_write("\r\nserial: %l",disk_info.serial);
		usart_write("\r\ndate:   %i/%i",disk_info.manufacturing_month,disk_info.manufacturing_year);
		usart_write("\r\nsize:   %l",disk_info.capacity);
		usart_write("\r\ncopy:   %i",disk_info.flag_copy);
		usart_write("\r\nwr.pr.: %i/%i",disk_info.flag_write_protect_temp,disk_info.flag_write_protect);
		usart_write("\r\nformat: %i",disk_info.format);
	}
	usart_write("\r\n");
}

//------------------------------------------------------------------------------
// Show filecontent

void command_catfile(void)
{
	int ch;
	char *string2=wortvar[0];
	File *ptrFile = f16_open(string2,"r");

	if (!ptrFile) usart_write("Read error: %s\r\n",string2);

	ch = f16_getc(ptrFile);
	while ( ch > 0 ) {
		usart_write_char((char)ch);
		ch = f16_getc(ptrFile);
	}
	f16_close(ptrFile);
}

//------------------------------------------------------------------------------
// delete file from SD-CARD

void command_delfile(void)
{
	char *string2=wortvar[0];
	
	if(f16_exist(string2)) {
		if (!f16_delete(string2)) 
			usart_write("%s kann nicht gelscht werden!\r\n",string2);
	} else {
		usart_write("Datei %s existiert nicht!\r\n",string2);
	}
}

//------------------------------------------------------------------------------
// show SD-Card directory

void command_dir(void)
{
	// usart_write("\r\nSD-Karte:");
	if (!cwdir_ptr)	usart_write(" Fehler beim Zugriff\r\n");
	else {
	
		struct fat16_dir_entry_struct dir_entry;
		#if FAT16_DATETIME_SUPPORT
		uint16_t year;
		uint8_t month,day,hour,min,sec;
		#endif
		char zeile[10];

		while(fat16_read_dir(cwdir_ptr, &dir_entry))
		{
			#if FAT16_DATETIME_SUPPORT
			fat16_get_file_modification_date(&dir_entry, &year, &month, &day);
			fat16_get_file_modification_time(&dir_entry, &hour, &min, &sec);

			usart_write("\r\n%2i.%2i.%i %2i:%2i ",day, month, year, hour, min);
			#else
			usart_write("\r\n");
			#endif
			if (dir_entry.attributes & FAT16_ATTRIB_DIR)
			{
				usart_write("<DIR>     ");
			} else {
				sprintf(zeile,"%9lu",dir_entry.file_size);
				usart_write("%s ",zeile);
			}
			usart_write("%s",dir_entry.long_name);
		}
	}
	usart_write("\r\n");
}

#endif	// SDCARD_COMMANDS

#if TWI_24Cxx
void command_writeSL24Cxx(void)
{
	SL24Cxx_write_byte(variable[0],variable[1]);
}

void command_readSL24Cxx(void)
{

	unsigned char inhalt;

	inhalt = SL24Cxx_read_byte(variable[0]);
	usart_write("Inhalt der Adresse[%2i] = %2x\r\n",variable[0],inhalt);
}
#endif

#if TWI_SERVO
void command_i2cservo(void)
{
	if (*((unsigned int*)&variable[1]) != 0x00000000)
	{	
		Servo_setposition(variable[0],variable[1]);
	}
	uint16_t spos = Servo_readposition(variable[0]);
	usart_write("Pos. Servo %i = %i\r\n",variable[0],spos);
}
#endif