/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:		 Anpassungen an AVR-NET-IO von Shortie
 known Problems: none
 Version:        08.01.2012
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_
	
	//-----------------------------------------------------------------------
	// MAC Adresse des Webservers
	//-----------------------------------------------------------------------
	
	#define MYMAC1	0x00	// Dies ist die Default MAC-Adresse von Pollin
	#define MYMAC2	0x22	// und sollte durch eigene Werte ersetzt werden.
	#define MYMAC3	0xf9	
	#define MYMAC4	0x07	
	#define MYMAC5	0x31	// Pollin hat fr das AVR-NET-IO Board eigene
	#define MYMAC6	0x84	// Adresse die dem Aufkleber zu entnehmen ist.
	
	// Bei Nutzung des TFTP Bootloaders sollten diese mit den Einstellungen
	// dort bereinstimmen.
	
	#define USERDEFINED_MAC	0	// damit kann MAC ber Console eingestellt werden
	
	// Bei USERDEFINED_MAC 1 wird auch vom TFTP-Bootloader nach erstmaliger 
	// Einstellung die eingestellte MAC-Adresse verwendet.

	//-----------------------------------------------------------------------
	// Umrechnung von IP zu unsigned long
	//-----------------------------------------------------------------------

	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//-----------------------------------------------------------------------
	// Netzwerkeinstellungen und Netzwerkdienste / DHCP
	//-----------------------------------------------------------------------

	#define MYIP					IP(192,168,8,9)	// IP des Webservers
    #define ROUTER_IP				IP(192,168,8,1)	// IP des Routers
	#define NETMASK					IP(255,255,255,0)	// Netzwerkmaske
	
    #define USE_DHCP				1	// 1 = DHCP Client on
  
	//-----------------------------------------------------------------------
	// HTTP-Server
	//-----------------------------------------------------------------------
	
	#define USE_HTTPD				1	// Webserver
	#define	HTTP_STATUSPAGE			1	// Ausgabe der Statuspage (0 spart Platz)
	#define	HTTP_CSVDATEN			1	// Ausgabe der Daten als CSV
	
	#define HTTP_REFRESH			1	// Refresh der Webseiten alle 60 Sekunden
	#define	HTTP_EMULATE_IE7		0	// IE7 Emulation fr IE8 und hher
	#define HTTP_VISITORS			1	// Besucherzhler auf Webseite

	#define HTTP_AUTH_DEFAULT		1	// Webserver mit Passwort? (0 = mit Passwort)
	#define HTTP_AUTH_STRING		"admin:password" // AUTH String "USERNAME:PASSWORT"
													 // max 14Zeichen

	#if HTTP_STATUSPAGE
		#define STATUSPAGE_REBOOT	0	// 1 = Neustart ber Webseite zulassen
	#endif
	
	//-----------------------------------------------------------------------
	// Pollin ADD-ON fr AVR-NET-IO-Board
	//-----------------------------------------------------------------------

	#define USE_ADDON_POLLIN		0	// Pollin ADD-ON Board benutzenreset
	
	
	#if USE_ADDON_POLLIN
		#define ADDON_IR_RC5		0	// Infrarotempfnger mit RC5-Codierung
										// Einstellungen in infrared/rc5.h beachten
		#define ADDON_LEDS			0	// LEDs auf ADD-ON
		#define ADDON_LCD			0	// LCD ber i2c
										// Einstellungen in i2clcd/i2clcd.h beachten
		#define ADDON_SDCARD		0	// SD-Karte auf ADD-ON nutzen
	#endif

	#if ADDON_LEDS	
		#define LED3_PWM			0	// Spielerei an LED 3 mit PWM (Timer2)
	#endif

	#if ADDON_SDCARD
		#define	SDCARD_LOGFILE		0	// Werte in Logfile schreiben
		#define SDCARD_COMMANDS		0	// SD-Karten-Befehle auf Console
	#endif

	//-----------------------------------------------------------------------
	// Schaltautomatik
	//-----------------------------------------------------------------------

	#define USE_SCHALTAUTOMATIK		1	// Relais 8 ber Automatik schalten
	
	#if USE_SCHALTAUTOMATIK
		volatile unsigned int automatikschalten;
	#endif

	//-----------------------------------------------------------------------
	// LCD einbinden an EXT-Port ohne ADD-ON Board
	//-----------------------------------------------------------------------

	#define USE_LCD_EXT				0	// LCD benutzen an EXT des AVR-NET-IO

	#if USE_LCD_EXT
		#define LCD_4Bit			0	// LCD im 4Bit Mode (1) oder seriell (0)
		#define LCD_4Bit_POLLIN		0	// wie LCD_4Bit mit Pollin Verdrahtung (Handbuch S.16)

		#define LCD_ONE_LINE		0	// 0 = mehrzeiliges Display / 1 = 1-zeiliges Display
		#define	LCD_4x20			0	// 0 = alles kleiner als 4x20 / 1 = 4x20
	#endif

	//-----------------------------------------------------------------------
	// LCD Hintergrundbeleuchtung und benutzerdefinierte Zeichen (USE_LCD_EXT und ADDON_LCD)
	//-----------------------------------------------------------------------

	#if(USE_LCD_EXT || ADDON_LCD)
		
		#define LCD_USER_CHARS		0	// sollen benutzerdefinierte Zeichen benutzt werden?
		#define	LCD_BACKLIGHT		0	// LCD hat Hintergrundbeleuchtung
	
		#if LCD_BACKLIGHT
			volatile unsigned char lcd_backlight;
			
			#define	LCD_BL_AUTO		0	// 1 = Hintergrundbeleuchtung automatisch abschalten
			#define LCD_BL_TIME		15	// Sekunden nach denen Beleuchtung abgeschaltet wird
			
			#if LCD_BL_AUTO
			volatile unsigned int lcd_bl_timer;
			#endif
		#endif
	#endif

	//-----------------------------------------------------------------------
	// NTP-Server einbinden - IP kann von DHCP berschrieben werden
	//-----------------------------------------------------------------------
	
	#define USE_NTP			1					// 1 = NTP Client on
	#define NTP_REFRESH 	1140				// NTP refresh
	//#define NTP_IP			IP(192,53,103,108)	// ptbtime1.ptb.de
	#define NTP_IP			IP(192,168,178,1)	// Fritz!Box
	//-----------------------------------------------------------------------
	// TELNET-Server			Anfnger lassen es bitte beides auf 0
	//-----------------------------------------------------------------------
	
	#define USE_TELNETD		1	// Telnetserver fr RS232 oder CMD_TELNET
    #define CMD_TELNET		1	// Commandos und Ausgaben erfolgen ber Telnet

	//-----------------------------------------------------------------------
    // DNS-Server einbinden - IP kann von DHCP berschrieben werden
	//-----------------------------------------------------------------------
	
    #define USE_DNS			0					// 1 = DNS Client on
    #define DNS_IP			IP(192,168,0,254)	// IP des DNS-Servers
    
	//-----------------------------------------------------------------------
	// SYSLOG-Server einbinden - IP kann im EEPROM berschrieben werden
	//-----------------------------------------------------------------------
	
	#define USE_SYSLOG		0					// 1 = SYSLOG on
	#define SYSLOG_IP		IP(192,168,0,10)	// IP des SYSLOG-Servers

	//-----------------------------------------------------------------------
	// Wake on Lan einbinden
	//-----------------------------------------------------------------------

	#define USE_WOL			0								// 1 = WOL on
	#define WOL_BCAST_IP	IP(192,168,0,255)				// Broadcast fr WOL
	#define WOL_MAC 		{0x00,0x40,0x63,0xD4,0xF0,0xCE}	// MAC die geweckt wird

	//-----------------------------------------------------------------------
	// Emailversand benutzen? Konfiguration des Emailclient in der sendmail.h
	//-----------------------------------------------------------------------

    #define USE_MAIL       	0
    
	//-----------------------------------------------------------------------
    // Daten ber HTTP-GET an Webserver senden
	//-----------------------------------------------------------------------

    #define USE_HTTP_CLIENT		0
	
	#if USE_HTTP_CLIENT
		#define HTTP_CLIENT_SERVER_IP	IP(192,168,0,10)
		#define HTTP_CLIENT_SERVERNAME	"www.meinwebserver.de"
		#define HTTP_CLIENT_SERVERPFAD	"/avr-net-io/daten.php"
	#endif

	//-----------------------------------------------------------------------
	// Benutze Werte aus EEPROM und Befehle fr EEPROM Werte
	//-----------------------------------------------------------------------
	
	#define USE_EEPVAR			1	// Startwerte / Sensoren-IDs aus EEPROM lesen
	#define USE_EEPVAR_CMD		1	// EEPVAR-Commands auf USART zulassen (0/1)
	
	//-----------------------------------------------------------------------
	// Kopplung mit 2. AVR-NET-IO ber UDP
	//-----------------------------------------------------------------------

	#define USE_UDPSEND			0
	
	#if USE_UDPSEND
		#define UDPSEND_RPORT	10011				// Ein Empfangs-Port muss beim Senden angegeben Werden.
		#define UDPSEND_SPORT	0					// Senden ( Ein ) wenn der Port != 0 ist
		#define UDPSEND_REFRESH	60					// Intervall in Sekunden fr Senden der Daten
		#define UDPSEND_IP		IP(192,168,0,91)	// IP des zweiten Boards (Intranet)
	#endif
	
	//-----------------------------------------------------------------------
	// Zustzliche Netzwerkkommandos
	//-----------------------------------------------------------------------
	
	#define USE_NETCMDS			0	// Schaltet PING, ARP, TCP und VER an
	
	//-----------------------------------------------------------------------
	// TFTP-Bootloader
	//-----------------------------------------------------------------------

	#define	USE_TFTPBOOT		0	// 0 = Nur Hardware Reset / 1 = REBOOT Command verfgbar

	#if USE_TFTPBOOT
		#define	TFTP_CMD		0	// 0 spart etwas Platz, aber keine Einstellungen auf der Console mglich
	#endif

	//-----------------------------------------------------------------------
	// 1-Wire Bus fr Temperatur-Sensoren
	//-----------------------------------------------------------------------
	
	#define USE_OW				0	// 1-Wire bus einbinden
	
	#if USE_OW
		#define OW_MAXSENSORS	8	// max. Anzahl der 1-wire sensoren (DS18x20)
		#define OW_EXT_POWER	1	// falls der 1-Wire Bus eine externe Versorgungsspannung hat

		#define OW_START		150	// Sensorstartwert 15,0 Grad in ow_array
		#define OW_MINMAX_START	0	// Min- / Maxwerte Startwert 0,0 Grad
		#define	OW_MINMAX		1	// Min- / Maxwerte speichern

		#define OW_PORTA		1	// 1-Wire Bus an PORTA7 *
		#define OW_PORTD		0	// 1-Wire Bus an PORTD3 und serielles LCD *
									
		// *) entweder OW_PORTA 1 oder OW_PORTD 1
		
		#undef	USE_EEPVAR
		#define USE_EEPVAR		1	// mu 1 sein da ROM-IDs im EEPROM gespeichert werden
	#endif

	//-----------------------------------------------------------------------
	// TWI/i2c Bus
	//-----------------------------------------------------------------------

	#define	USE_TWI				1	// soll TWI/i2c benutzt werden?
			volatile unsigned int zeith;	//Stunden
			volatile unsigned int zeitm;	//Minuten
			volatile unsigned int zeits;	//Sekunden
			volatile unsigned int zeitt;	//Tag
			volatile unsigned int zeitd;	//Monat
			volatile unsigned int zeitw;	//Wochentag
	#define adresse 		var_array[31]	// device address whlbar
	
	#if USE_TWI
		#define	TWI_SERVO		0	// ATtiny2313 i2c-Servocontroller
		#define	TWI_24Cxx		0	// externes i2c EEPROM SL24Cxx benutzen
		#define adresse1		0xAE	// device address Dimmer 174
		#define adresse2		0xB0	// device address Dimmer 176
		#define adresse3		0xB2	// device address Dimmer 178
	#endif							// Wichtig: i2c/24Cxx.h beachten!

//===========================================================================
// Voreinstellungen AVR-NET-IO
// ab hier nur ndern wenn man wei was man macht!
//===========================================================================

	//-----------------------------------------------------------------------
	// Konfiguration der PORTS (HEX)		1=OUTPUT / 0=INPUT
	//-----------------------------------------------------------------------
	
	#define OUTA	0x00	// PORTA sind Eingnge
	#define OUTC	0xFF	// PORTC sind Ausgnge
	#define OUTD	0xFC	// PORTD 6x Ausgang + 2 Pins fr RS232

	//Achtung!!!! an PORTB ist der ENC -  nur ndern wenn man wei was man macht!
	
	//-----------------------------------------------------------------------
	// Taktfrequenz
	//-----------------------------------------------------------------------

    #define F_CPU 16000000	
	
	//-----------------------------------------------------------------------
	// Baudrate der seriellen Schnittstelle
	//-----------------------------------------------------------------------

	#define BAUDRATE 9600

	//-----------------------------------------------------------------------
	// AD-Wandler benutzen
	//-----------------------------------------------------------------------
	
	#define USE_ADC			1
	#define	ADC_START		4	// ab welchem Kanal (0-7) der ADC genutzt wird
								// 0 => A0-A3 auch als analoge Eingnge
								// 4 => A4-A7 als analoge Eingnge (default)

	//-----------------------------------------------------------------------
	// Variablen Array
	//-----------------------------------------------------------------------

	#define MAX_VAR_ARRAY	100
	
	//-----------------------------------------------------------------------
	// Speicherstellen im EEPROM
	//-----------------------------------------------------------------------

	#define IP_EEPROM_STORE			30	// IP Adresse *
	#define NETMASK_EEPROM_STORE	34	// Netzmaske *
	#define ROUTER_IP_EEPROM_STORE	38	// IP Gateway *
	#define DNS_IP_EEPROM_STORE		42	// IP Adresse DNS *
	//#define ??_IP_EEPROM_STORE	46	// 4 Byte **
	#define NTP_IP_EEPROM_STORE 	50	// IP Adresse NTP-Server *
	#define MAC_EEPROM_STORE		54	// 6 Byte
	#define WOL_MAC_EEPROM_STORE	60	// Wake On Lan MAC-Adresse
	#define WOL_BCAST_EEPROM_STORE	66	// Wake On Lan Broadcast-Adresse
	#define SYSLOG_IP_EEPROM_STORE	70	// IP Adresse SYSLOG-Server
	#define UDPSEND_IP_EEPROM_STORE	74	// IP des 2. Boards im EEPROM **
	#define UDPSEND_RP_EEPROM_STORE	78	// Receive-Port im EEPROM **
	#define UDPSEND_SP_EEPROM_STORE	80	// Send-Port im EEPROM **
	
	#define TFTP_IP_EEPROM_STORE	108	// IP des TFTP-Server
	#define TFTP_FILE_EEPROM_STORE	112	// Dateiname Hex-Datei (16 Byte)
	#define DEFPC_EEPROM_STORE		128	// Starteinstellung PORTC
	#define DEFPD_EEPROM_STORE		129	// Starteinstellung PORTD

	#define SV_EEPROM_STORE			160	// var_array Speicherung im EEPROM
	#define OW_EEPROM_STORE 		640	// 1-Wire Sensoren

	//   *)	Werte in diesen Stellen werden bei DHCP bergangen.
	//  **)	derzeit ungenutzt
	
	//-----------------------------------------------------------------------
	// Voreinstellungen um Konflikte aufzulsen
	//-----------------------------------------------------------------------

	#if USE_LCD_EXT					// LCD am Ext.Port
		#if LCD_4Bit_POLLIN
			#undef	LCD_4Bit
			#define	LCD_4Bit		1
		#endif
		#if LCD_4Bit				// LCD direkt angeschlossen
			#undef	ADDON_IR_RC5
			#define	ADDON_IR_RC5	0	// LCD hat Vorrang - kein Infrarot an PB0
			#undef	ADDON_LEDS
			#define	ADDON_LEDS		0	// LCD hat Vorrang - keine LEDs auf ADD-ON
			#undef	LED3_PWM
			#define LED3_PWM		0
			#if ADDON_SDCARD
				#undef	LCD_BACKLIGHT
				#define	LCD_BACKLIGHT	0	// SD-Card hat Vorrang vor LCD_Backlight
			#endif
		#else							// LCD am Ext.Port seriell
			#undef	ADDON_LEDS
			#define	ADDON_LEDS		0	// LCD hat Vorrang - keine LEDs auf ADD-ON
			#undef	LED3_PWM
			#define LED3_PWM		0
		#endif
	#endif

	#if ADDON_LCD
		#undef	USE_TWI
		#define	USE_TWI				1	// LCD auf ADDON-Board nutzt TWI/i2c
	#endif

	//---------------------------------------------------------------------------
	#define URVersion       "1.2.5"		// Ulrich Radigs Original Version
	#define Version         "1.2.8"		// Diese Version

	extern unsigned int var_array[MAX_VAR_ARRAY];
	#if USE_UDPSEND
	extern unsigned int va2_array[MAX_VAR_ARRAY];
	#endif
	
#endif //_CONFIG_H
