/*----------------------------------------------------------------------------
 Copyright:      Chris Fiege    < Chris.Fiege@web.de >
 Author:         Chris Fiege
 Remarks:        angepasst von Shortie
 known Problems: none
 Version:        24.01.2011
 Description:    http_client

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
---------------------------------------------------------------------------- 
 
 Lizenzbedingung angepasst auf GPLv2 nach Rcksprache mit Chris Fiege
 
----------------------------------------------------------------------------*/


#include "config.h"
#include "http_client.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <stdlib.h>

#include "stack.h"
#include "usart.h"
#include "timer.h"
#include "1-wire/messung.h"


//Konstanten in Programmspeicher, die gebraucht werden um die HTTP-GET Anfrage zusammenzusetzen
PROGMEM char HTTP_CLIENT_GET_START[] = {"GET "HTTP_CLIENT_SERVERPFAD"?"};
PROGMEM char HTTP_CLIENT_GET_END[]   = {" HTTP/1.1\r\n"
									  "Host: "HTTP_CLIENT_SERVERNAME"\r\n\r\n"};
PROGMEM char HTTP_CLIENT_GET_ID[]    = {"&id[]="};
PROGMEM char HTTP_CLIENT_GET_VAL[]   = {"&val[]="};


//State- und Counter-Variablen fr die Folgende State-Maschine
char HTTP_CLIENT_state = 0;
int  HTTP_CLIENT_counter = 0;
int HTTP_CLIENT_counter1 = 0;

char HTTP_CLIENT_portcount = 0;

int HTTP_CLIENT_success = 0;
int HTTP_CLIENT_tries = 0;

//Auslesen und Auswerten der HTTP-Response des Servers
char HTTP_CLIENT_wandler[16];
const char HTTP_CLIENT_HTTP_OK[] = {"HTTP/1.1 200 OK"};


void http_client_data (unsigned char tcp_con_index) {
	//Behandle die eingehenden Daten der TCP-Sitzung
	//tcp_con_indel ist der Index der TCP-Verbindung, die dieser Aufruf betrifft
	
	if(HTTP_CLIENT_state == 4) {
		//Wir sind im Daten-Empfangen state.
		//Wir sollten die Daten also auswerten!
		if((TCP_DATA_END_VAR - TCP_DATA_START_VAR) > 0 || HTTP_CLIENT_counter < 15) {
			//Kopieren wir die Daten in den Puffer
			if((TCP_DATA_END_VAR - TCP_DATA_START_VAR) >= (15-HTTP_CLIENT_counter)) {
				//Es sind mehr Daten gekommen als wir brauchen. kopieren wir uns doch alle
				memcpy(&HTTP_CLIENT_wandler[HTTP_CLIENT_counter], &eth_buffer[TCP_DATA_START_VAR],(15-HTTP_CLIENT_counter));
				HTTP_CLIENT_counter = 15;
			} else {
				//Es sind weniger Daten angekommen. Kopieren wir was wir haben ;-)
				memcpy(&HTTP_CLIENT_wandler[HTTP_CLIENT_counter], &eth_buffer[TCP_DATA_START_VAR],(TCP_DATA_END_VAR - TCP_DATA_START_VAR));
				HTTP_CLIENT_counter = (TCP_DATA_END_VAR - TCP_DATA_START_VAR);
			}
		}
		
		//Testweise: Ausgabe der Antwort:
		/*for (int a = TCP_DATA_START_VAR;a < TCP_DATA_END_VAR;a++)
		{
			HTTP_CLIENT_DEBUG("%c",eth_buffer[a]);
		} */
		
		//TCP-ACK senden um weitere Daten Empfangen zu knnen:
		tcp_entry[tcp_con_index].time = TCP_TIME_OFF;
		tcp_entry[tcp_con_index].status = ACK_FLAG;
		create_new_tcp_packet(0,tcp_con_index);
	}
	
}


void http_client_do(void) {
	int tcp_con_index = MAX_ARP_ENTRY;
	int i;
	
	unsigned char mm = (time/60)%60;
	
	switch(HTTP_CLIENT_state) {
	case 0:
		//Initialisierung des HTTP-Client
		HTTP_CLIENT_DEBUG("HC 0: Initialisiere!\r\n");
		
		//Fge den HTTP-Client der Application-Liste des Stack hinzu:
		//Uns "gehrt" der Port HTTP_CLIENT_PORT (siehe http_client.h)
		for(i=0; i<HTTP_CLIENT_PORT_RANGE; i++) {
			add_tcp_app (HTTP_CLIENT_PORT+i, (void(*)(unsigned char))http_client_data);
		}
		
		HTTP_CLIENT_state = 90;
		
	break;
	
	case 1:
		//ARP-Anfrage und Anfrage der TCP-Verbindung
		HTTP_CLIENT_DEBUG("HC 1: ARP-Request\r\n");
		HTTP_CLIENT_tries++;
		
		if(!arp_request(HTTP_CLIENT_SERVER_IP)) {
			//ARP-Anfrage nach der HTTP_CLIENT_SERVER_IP ist fehlgeschlagen
			//Da noch keine TCP-Verbindung aufgebaut wurde knnen wir einfach in den Warte-Zustand wechseln
			HTTP_CLIENT_DEBUG("HC 1: ARP-Request fehlgeschlagen\r\n");
			HTTP_CLIENT_state = 90;
		} else {
			//ARP-Anfrage nach der HTTP_CLIENT_SERVER_IP war erfolgreich.
			//Nun knnen wir versuchen eine TCP-Verbindung dahin zu ffnen!
			tcp_port_open (HTTP_CLIENT_SERVER_IP,HTONS(80),HTONS((HTTP_CLIENT_PORT+HTTP_CLIENT_portcount)));
			HTTP_CLIENT_DEBUG("HC 1: TCP-Verbindung angefragt auf Port %i\r\n",(HTTP_CLIENT_PORT+HTTP_CLIENT_portcount));
			if((++HTTP_CLIENT_portcount) == HTTP_CLIENT_PORT_RANGE) HTTP_CLIENT_portcount=0;
			
			
			//Wechsel in State 2. Lade Counter auf
			HTTP_CLIENT_counter = 1000;
			HTTP_CLIENT_state = 2;
		}
	break;
	
	case 2:
		//Warte darauf, dass eine TCP-Verbindung hergestellt wird
		//Timeout beachten
		//HTTP_CLIENT_DEBUG("HC 2: Warte auf TCP-Verbindung\r\n");
		
		//Suche die Verbindung im Stack:
		tcp_con_index = tcp_entry_search (HTTP_CLIENT_SERVER_IP,HTONS(80));
		
		if(tcp_entry[tcp_con_index].app_status == 1) {
			//Falls die Verbindung hergestellt wurde:
			//Sende ein Ack und welchsel in den nchsten State:
			HTTP_CLIENT_DEBUG("HC 2: Hergestellt! (%i)\r\n",HTTP_CLIENT_counter);
			tcp_entry[tcp_con_index].first_ack = 1;
			HTTP_CLIENT_state = 3;
		}
		
		if(HTTP_CLIENT_counter-- == 0) {
			//Verbindung wurde nicht rechtzeitig hergestellt
			//Beenden wir die Verbindung sicherheitshalter und schlieen wir diesen Vorgang
			HTTP_CLIENT_state = 5;
			HTTP_CLIENT_DEBUG("HC 2: Nicht erfolgreich\r\n");
		}
	break;
	
	case 3:
		//TCP-Verbindung wurde erfolgreich aufgebaut
		//Setze den HTTP-GET Header zusammen und sende ihn an den Server
		HTTP_CLIENT_DEBUG("HC 3: TCP-Verbindung erfolgreich.\r\n");
		
		//Hole schon mal die ID fr unsere TCP-Verbindung
		tcp_con_index = tcp_entry_search (HTTP_CLIENT_SERVER_IP,HTONS(80));
		
		//Gibt an, wie viel Byte wir bereits in den TCP-Buffer kopiert haben
		int buffersize = 0;
		
		//Kopiere Anfang der HTTP-Get Anfrage in den TCP-Buffer
		memcpy_P(&eth_buffer[TCP_DATA_START], HTTP_CLIENT_GET_START, sizeof(HTTP_CLIENT_GET_START)-1);
		buffersize += sizeof(HTTP_CLIENT_GET_START)-1;
		
		//Kopiere jetzt fr jeden Sensor, der bermittelt werden soll:
		for(i=0; i<HTTP_CLIENT_NUM_OW_SENS; i++) {
			//Den konstanten Teil der ID:
			memcpy_P(&eth_buffer[TCP_DATA_START+buffersize],HTTP_CLIENT_GET_ID, sizeof(HTTP_CLIENT_GET_ID)-1);
			buffersize += sizeof(HTTP_CLIENT_GET_ID)-1;
			
			//Den Dynamischen Teil der ID
			//Dazu mssen wir die ID erstmal wandeln:
			itoa(i+HTTP_CLIENT_OW_SENS_OFFSET,HTTP_CLIENT_wandler,10);
			//Nun knnen wir den erzeugten String in den Ausgabepuffer kopieren
			memcpy(&eth_buffer[TCP_DATA_START+buffersize], HTTP_CLIENT_wandler, strlen(HTTP_CLIENT_wandler));
			buffersize += strlen(HTTP_CLIENT_wandler);
			
			//Den konstanten Teil des Wertes:
			memcpy_P(&eth_buffer[TCP_DATA_START+buffersize],HTTP_CLIENT_GET_VAL, sizeof(HTTP_CLIENT_GET_VAL)-1);
			buffersize += sizeof(HTTP_CLIENT_GET_VAL)-1;
			
			//Den Dynamischen Teil des Wertes
			//Dazu mssen wir die ID erstmal wandeln:
			itoa(ow_array[i]&0xFFFF,HTTP_CLIENT_wandler,10);
			//Nun knnen wir den erzeugten String in den Ausgabepuffer kopieren
			memcpy(&eth_buffer[TCP_DATA_START+buffersize], HTTP_CLIENT_wandler, strlen(HTTP_CLIENT_wandler));
			buffersize += strlen(HTTP_CLIENT_wandler);
		}
		
		//Kopiere Anfang der HTTP-Get Anfrage in den TCP-Buffer
		memcpy_P(&eth_buffer[TCP_DATA_START+buffersize], HTTP_CLIENT_GET_END, sizeof(HTTP_CLIENT_GET_END)-1);
		buffersize += sizeof(HTTP_CLIENT_GET_END)-1;
		
		/*
		//Anfrage fertig gebaut
		//Testweise: Ausgabe ;-)
		eth_buffer[TCP_DATA_START+buffersize] = 0;
		HTTP_CLIENT_DEBUG("%s",&eth_buffer[TCP_DATA_START]);
		*/
		
		
		//Bereite das TCP-Paket vor:
		tcp_entry[tcp_con_index].status = ACK_FLAG | PSH_FLAG;
		
		//Absenden!
		create_new_tcp_packet(buffersize,tcp_con_index);
		
		//In den nchsten Zustand wechseln!
		HTTP_CLIENT_state = 4;
		HTTP_CLIENT_counter = 0;
		HTTP_CLIENT_counter1 = 10000;
	break;
	
	case 4:
		//Warte auf Empfangene Daten:
		//HTTP_CLIENT_DEBUG("HC 4: Bytes empfangen: %i\r\n",HTTP_CLIENT_counter);
		if(HTTP_CLIENT_counter1-- == 0) {
			//Empfangs-timeout ist abgelaufen, bevor wir daten Empfangen haben.
			//Wir beenden die Verbindung und versuchen es spter noch einmal!
			HTTP_CLIENT_DEBUG("HC 4:Keine Daten vor Timeout empfangen!\r\n");
			HTTP_CLIENT_state =5;
			
		} else if(HTTP_CLIENT_counter >= 15) {
			//Es wurden 15 Bytes Daten empfangen und es ist noch kein Timeout passiert:
			
			//Um die Empfangene Zeile ausgeben zu knnen fgen wir an die 16. Position (letztes char) eine \0 ein!
			HTTP_CLIENT_wandler[15] = 0;
			//Ausgabe als wr es ein String (ist es jetzt ja auch ;) )
			HTTP_CLIENT_DEBUG("HC 4:->%s<-\r\n",HTTP_CLIENT_wandler);
			
			//Prfe, ob dies eine erfolgreiche Anfrage war:
			//Gehen wir davon aus, dass die Anfrage erfolgreich war und beweisen mit dem Vergleich im zweifel das Gegenteil!
			char merker = 1;
			for(i=0; i<15; i++) {
				//Fr jedes Zeichen in beiden Strings:
				if(HTTP_CLIENT_wandler[i] != HTTP_CLIENT_HTTP_OK[i]) merker = 0;
			}
			//Prfe, ob es erfolgreich war:
			if(merker) {
				//Erfolgreich:
				HTTP_CLIENT_DEBUG("HC 4: Erfolgreich! (%i)\r\n",HTTP_CLIENT_counter1);
				++HTTP_CLIENT_success;
			} else {
				//Oder eben nicht:
				HTTP_CLIENT_DEBUG("HC 4: Nicht erfolgreich! (%i)\r\n",HTTP_CLIENT_counter1);
			}			
			HTTP_CLIENT_state = 5;
		}
	break;
	
	case 5:
		//Verbindung beenden
		HTTP_CLIENT_DEBUG("HC 5: TCP-Verbindung schlieen\r\n");
		HTTP_CLIENT_DEBUG("HC 5: OK: %i / Gesamt: %i\r\n",HTTP_CLIENT_success, HTTP_CLIENT_tries);
		
		//Zum Beenden brauchen wir den index der Verbindung
		tcp_con_index = tcp_entry_search (HTTP_CLIENT_SERVER_IP,HTONS(80));
		//TCP-Verbindung beenden
		tcp_Port_close (tcp_con_index);
		
		//In den Wartezustand wechseln
		HTTP_CLIENT_state = 90;
	break;
	
	case 90:
		//Warte auf nchste Minute!
		HTTP_CLIENT_counter = mm;
		HTTP_CLIENT_state = 91;
	break;
	
	case 91:
		if(HTTP_CLIENT_counter != mm) {
			HTTP_CLIENT_state = 1;
		}
	}
	
}