/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        angepasst von Shortie
 known Problems: none
 Version:        08.01.2012
 Description:    Webserver Applikation

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "config.h"

#if USE_HTTPD

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "stack.h"
#include "base64.h"
#include "timer.h"
#include "cmd.h"
#include "usart.h"

#if HTTP_VISITORS
	#define BESUCHER	27		// var_array[27] ist Besucherzhler
#endif

#if USE_ADC
	#include "analog.h"
#endif

#if USE_LCD_EXT
	#include "lcd.h"
#endif

#if USE_WOL
	#include "wol.h"
#endif

#if USE_NTP
	#include "ntp.h"
#endif

#if USE_OW
	#include "1-wire/messung.h"
#endif

#if HTTP_STATUSPAGE
	#if USE_DNS
		#include "dnsc.h"
	#endif
	#if TFTP_CMD
		unsigned char tftp_server_ip[4];
	#endif
	#if USE_SYSLOG
		#include "syslog/syslog.h"
	#endif
	#include "networkcard/enc28j60.h"
#endif

#if USE_TWI
	#include "i2c/i2cmaster.h"
#endif

#include "httpd.h"
#include "webtexte.h"
#include "webpage-netio.h"

unsigned int wert = 0;
unsigned int n = 0;
unsigned char ret;			//fr Abfrage i2c bereit
unsigned int i2cout = 0;
//===========================================================================

struct http_table http_entry[MAX_TCP_ENTRY];

//Hier wird das codierte Passwort aus config.h gespeichert.
unsigned char http_auth_passwort[30];

unsigned char post_in[5] = {'O','U','T','='};
unsigned char post_inval[5] = {'I','N','V','A'};
unsigned char post_ready[5] = {'S','U','B','='};
unsigned char PORT1_tmp = 0;
unsigned char PORT1_off = 0;
unsigned char PORT1_write = 0;

char dstr[24]={"No Time...             "};

#if USE_SCHALTAUTOMATIK
unsigned int inval = 0;
unsigned int zeiten_speichern[10];	// Anzahl der max. Schaltzeiten
 unsigned int wochentag_speichern[8];			// Bedingung + Montag-Sonntag 
 unsigned char wert_tmp = 0;			// 				"						zu 1Byte
 unsigned int wochentag_marker;	//Merker zum Apeichern
#endif

//----------------------------------------------------------------------------

//TEXT_PLAIN
PROGMEM char http_header0[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR-NET-IO\r\n"
								"Content-Type: text/plain\r\n\r\n"};
//TEXT_HTML
PROGMEM char http_header1[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR-NET-IO\r\n"
								"Content-Type: text/html\r\n\r\n"};
//TEXT_HTML_AUTH
PROGMEM char http_header2[]={	"HTTP/1.0 401 Unauthorized\r\n"
								"Server: AVR-NET-IO\r\n"
								"WWW-Authenticate: Basic realm=\"NeedPassword\""
								"\r\nContent-Type: text/html\r\n\r\n"};
//TEXT_CSS						
PROGMEM char http_header3[]={	"HTTP/1.0 200 Document follows\r\n"
								"Server: AVR-NET-IO\r\n"
								/*"Last-Modified: Fri, 08 Jul 2008 16:48:29 GMT"*/
								"Cache-Control: max-age=6000000"
								/*"Expires: Fri, 08 Jul 2010 16:48:29 GMT"*/
								"Content-Type: text/css\r\n\r\n"};



//----------------------------------------------------------------------------
//Kein Zugriff Seite bei keinem Passwort
PROGMEM char Page0[] = {"401 Unauthorized%END"};

unsigned char rx_header_end[5] = {"\r\n\r\n\0"};

//----------------------------------------------------------------------------
//Initialisierung des Httpd Testservers
void httpd_init (void)
{
	//HTTP_AUTH_STRING 
	decode_base64((unsigned char*)HTTP_AUTH_STRING,http_auth_passwort);

	//Serverport und Anwendung eintragen
	add_tcp_app (HTTPD_PORT, (void(*)(unsigned char))httpd);
}
   
//----------------------------------------------------------------------------
//http Testserver
void httpd (unsigned char index)
{
    //Verbindung wurde abgebaut!
    if (tcp_entry[index].status & FIN_FLAG)
    {
        return;
    }

	//Allererste Aufruf des Ports fr diese Anwendung
	//HTTPD_Anwendungsstack lschen
	if(tcp_entry[index].app_status==1)
	{
		httpd_stack_clear(index);
	}
	
	//HTTP wurde bei dieser Verbindung zum ersten mal aufgerufen oder
	//HTML Header Retransmission!
	if (tcp_entry[index].app_status <= 2)
	{	
		httpd_header_check (index);
		return;
	}
	
	//Der Header wurde gesendet und mit ACK besttigt (tcp_entry[index].app_status+1)
	//war das HTML Packet fertig, oder mssen weitere Daten gesendet werden, oder Retransmission?
	if (tcp_entry[index].app_status > 2 && tcp_entry[index].app_status < 0xFFFE && tcp_entry[index].status == ACK_FLAG)
	{
		httpd_data_send (index);
		return;
	}
	
	//Verbindung kann geschlossen werden! Alle HTML Daten wurden gesendet TCP Port kann
	//geschlossen werden (tcp_entry[index].app_status >= 0xFFFE)!!
	if (tcp_entry[index].app_status >= 0xFFFE)
	{
		tcp_entry[index].app_status = 0xFFFE;
		tcp_Port_close(index);
		return;
	}
	return;
}

//----------------------------------------------------------------------------
//HTTPD_STACK lschen
void httpd_stack_clear (unsigned char index)
{
	http_entry[index].http_header_type = TEXT_HTML;
	http_entry[index].first_switch = 0;
	http_entry[index].http_auth = HTTP_AUTH_DEFAULT;
	http_entry[index].new_page_pointer = 0;
	http_entry[index].old_page_pointer = 0;
	http_entry[index].post = 0;
	http_entry[index].auth_ptr = http_auth_passwort;
	http_entry[index].post_ptr = post_in;
	http_entry[index].post_inval = post_inval;
	http_entry[index].post_ready_ptr = post_ready;
	http_entry[index].hdr_end_pointer = rx_header_end;
			
	HTTP_DEBUG("\r\n**** NEUE HTTP ANFORDERUNG ****\r\n\r\n");	
	return;
}

//----------------------------------------------------------------------------
//Eintreffenden Header vom Client checken
void httpd_header_check (unsigned char index)
{
	unsigned int a = 0;
	
	if(strcasestr_P((char*)&eth_buffer[TCP_DATA_START_VAR],PSTR("POST"))!=0)
	{
		http_entry[index].post = 1;
	}
	
	//finden der Authorization und das Ende im Header auch ber mehrere Packete hinweg!!	
	if(*http_entry[index].hdr_end_pointer != 0)
	{		
		for(a=TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{	
			HTTP_DEBUG("%c",eth_buffer[a]);
			
			if(!http_entry[index].http_auth) 
			{
				if (eth_buffer[a] != *http_entry[index].auth_ptr++)
				{
					http_entry[index].auth_ptr = http_auth_passwort;
				}
				if(*http_entry[index].auth_ptr == 0) 
				{
					http_entry[index].http_auth = 1;
					HTTP_DEBUG("  <---LOGIN OK!--->\r\n");
				}
			}
			
			if (eth_buffer[a] != *http_entry[index].hdr_end_pointer++)
			{
				http_entry[index].hdr_end_pointer = rx_header_end;
			}
			
			//Das Headerende wird mit (CR+LF+CR+LF) angezeigt!
			if(*http_entry[index].hdr_end_pointer == 0) 
			{
				HTTP_DEBUG("<---HEADER ENDE ERREICHT!--->\r\n");
				break;
			}
		}
	}
	
	
	//Welche datei wird angefordert? Wird diese in der Flashspeichertabelle gefunden?
	unsigned char page_index = 0;
	
	if (!http_entry[index].new_page_pointer)
	{
		for(a = TCP_DATA_START_VAR+5;a<(TCP_DATA_END_VAR);a++)
		{
			if (eth_buffer[a] == '\r')
			{
				eth_buffer[a] = '\0';
				break;
			}
		}
	
		// Initialize pointer to avoid problems when communcation is not finished after webpage transmssion
		http_entry[index].new_page_pointer = 0;
		
		while(WEBPAGE_TABLE[page_index].filename)
		{
			if (strcasestr((char*)&eth_buffer[TCP_DATA_START_VAR],WEBPAGE_TABLE[page_index].filename)!=0) 
				{
					HTTP_DEBUG("\r\n\r\nDatei gefunden: ");
					HTTP_DEBUG("%s",(char*)WEBPAGE_TABLE[page_index].filename);
					HTTP_DEBUG("<----------------\r\n\r\n");	
					
					if (strcasestr(WEBPAGE_TABLE[page_index].filename,".htm")!=0)
					{
						http_entry[index].http_header_type = TEXT_HTML;	
					}
					else if (strcasestr(WEBPAGE_TABLE[page_index].filename,".css")!=0)
					{
						http_entry[index].http_header_type = TEXT_CSS;	
					}
					else
					{
						http_entry[index].http_header_type = TEXT_PLAIN;
					}

					http_entry[index].new_page_pointer = WEBPAGE_TABLE[page_index].page_pointer;
					break;
				}
			page_index++;
		}
	}

	//Einzelne Postpacket (z.B. bei firefox)
	if(http_entry[index].http_auth && http_entry[index].post == 1)
	{
		HTTP_DEBUG("\r\nPage_index: %i\r\n",page_index);

		for(a = TCP_DATA_START_VAR;a<(TCP_DATA_END_VAR);a++)
		{

			//---------------------------------------------------------------
			// Numerische Textfelder finden
			//---------------------------------------------------------------

			#if USE_SCHALTAUTOMATIK
			if (eth_buffer[a] != *http_entry[index].post_inval++)
			{
				http_entry[index].post_inval = post_inval;
			}
			if(*http_entry[index].post_inval == 0) 
			{
				switch (page_index)
				{
				
//-----------------------------------------------				
				case(0):	// index.htm
					inval = (eth_buffer[a+1] & 0b00001111)*10;
					inval = inval+(eth_buffer[a+2] & 0b00001111);
						
					
							if (((eth_buffer[a+4]) > 47) & ((eth_buffer[a+4]) < 58))
								{
								wert = (eth_buffer[a+4] & 0b00001111);
								}
								
							if (((eth_buffer[a+5]) > 47) & ((eth_buffer[a+5]) < 58))	
								{
								wert = (wert *10) + (eth_buffer[a+5] & 0b00001111);
								}
								
							if (((eth_buffer[a+6]) > 47) & ((eth_buffer[a+6]) < 58))	
								{
								wert = (wert *10) + (eth_buffer[a+6] & 0b00001111);
								}
								
							if (wert > 255)
								{wert = 255 ;}	//gltiger Wert
								
						if (inval == 28)	
							{
								if (wert == 1) {var_array[31]=adresse1; var_array[28]=1;}
								if (wert == 2) {var_array[31]=adresse2; var_array[28]=2;}
								if (wert == 3) {var_array[31]=adresse3; var_array[28]=3;}
							}

						
						if (inval == 29)	
							{
								var_array[29]=wert;
									var_array[30]=(var_array[29]-1)*6;
							}
												
						if 	(var_array[29]>0)
						{
								if (inval > 29)  //1 Byte
							{
					ret = i2c_start(var_array[31]+I2C_WRITE);
								if ( ret ) 
								{}
								else 
								{
								i2c_start_wait(var_array[31]+I2C_WRITE);     // set device address and write mode
								i2c_write((inval)+((var_array[29]-1)*6));                      // Schaltzeituhr
								i2c_write(wert);	//	wert				
								i2c_stop();	
								}
							}
			
						}
								
								
						if (inval < 30)	{		//1 Byte an Mischer
							n = inval;
							ret = i2c_start(var_array[31]+I2C_WRITE);
							if ( ret ) 
							{
							}else 
							{
							i2c_start_wait(var_array[31]+I2C_WRITE);     // set device address and write mode
							i2c_write(n);                      // 
							i2c_write(wert);	//	wert				
							i2c_stop();		
							}
						}			
				
				break;
				
//-----------------------------------------------
				
				
				
				
				
				case(2):	// autom.htm
					inval = (eth_buffer[a+1] & 0b00001111)*10;
					inval = inval+(eth_buffer[a+2] & 0b00001111);					
					var_array[inval] = (eth_buffer[a+4] & 0b00001111)*10;
					var_array[inval] = var_array[inval]+(eth_buffer[a+5] & 0b00001111);

					automatikschalten = 1;

				}
			}
			#endif
			
			//---------------------------------------------------------------
			// Schaltanweisung finden!
			//---------------------------------------------------------------

			if (eth_buffer[a] != *http_entry[index].post_ptr++)
			{
				http_entry[index].post_ptr = post_in;
			}
			if(*http_entry[index].post_ptr == 0) 
			{
				switch (page_index)
				{
				case(0):	// index.htm
				
					switch(eth_buffer[a+1])
					{
						#if !USE_TWI
						case ('A'):
							PORT1_tmp = PORT1_tmp + 1;
							if (var_array[11]& 0x01 ) PORT1_off = PORT1_off + 1;
							break;
	
						case ('B'):
							PORT1_tmp = PORT1_tmp + 2;
							if (var_array[11]& 0x02 ) PORT1_off = PORT1_off + 2;
							break;
						#endif
						case ('C'):
							PORT1_tmp = PORT1_tmp + 4;
							if (var_array[11]& 0x04 ) PORT1_off = PORT1_off + 4;
							break;

						case ('D'):
							PORT1_tmp = PORT1_tmp + 8;
							if (var_array[11]& 0x08 ) PORT1_off = PORT1_off + 8;
							break;
	
						case ('E'):
							PORT1_tmp = PORT1_tmp + 16;
							if (var_array[11]& 0x10 ) PORT1_off = PORT1_off + 16;
							break;
	
						case ('F'):
							PORT1_tmp = PORT1_tmp + 32;
							if (var_array[11]& 0x20 ) PORT1_off = PORT1_off + 32;
							break;

						case ('G'):
							PORT1_tmp = PORT1_tmp + 64;
							if (var_array[11]& 0x40 ) PORT1_off = PORT1_off + 64;
							break;
		
						case ('H'):
							PORT1_tmp = PORT1_tmp + 128;
							if (var_array[11]& 0x80 ) PORT1_off = PORT1_off + 128;
							break;
			
						case ('Y'):
							PORT1_write = 1;
							break;
							
						case ('Z'):				//Uhr stellen
						ret = i2c_start(var_array[31]+I2C_WRITE);
						if ( ret ) 
						{
						}else 
						{
						if (zeitw == 0) {zeitw = 7;}			//Sonntag soll 7 sein und nicht 0
							i2c_start_wait(var_array[31]+I2C_WRITE);     // set device address and write mode
						//	i2c_write(23);                      // erste Schreibadresse
							i2c_write(20);                      // erste Schreibadresse
							i2c_write(zeitm);   // Minute
							i2c_write(zeith);   // Stunde
							i2c_write(zeitt);   // Tag
							i2c_write(zeitd);   //	Monat
							i2c_write(zeitw);	// Wochentag						
							i2c_stop();		
						}
						break;	
								
						case ('a'):	
							wochentag_speichern[7] = 1;
							break;
	
						case ('b'):
							wochentag_speichern[0] = 1;
							break;
						case ('c'):
							wochentag_speichern[1] = 1;
							break;

						case ('d'):
							wochentag_speichern[2] = 1;
							break;
	
						case ('e'):
							wochentag_speichern[3] = 1;
							break;
	
						case ('f'):
							wochentag_speichern[4] = 1;
							break;

						case ('g'):
							wochentag_speichern[5] = 1;
							break;
		
						case ('h'):
							wochentag_speichern[6] = 1;
							break;
	
						#if USE_WOL
						case 'W':
							wol_enable = 1;
							break;
						#endif //USE_WOL
					}
					break;	// Ende page_index 0
				
				#if HTTP_STATUSPAGE
				case(1):	// status.htm
				
					switch(eth_buffer[a+1])
					{
					#if STATUSPAGE_REBOOT
						case ('A'):
							command_reset();
							break;
						#if USE_TFTPBOOT
						case ('B'):
							command_reboot();
							break;
						#endif	// USE_TFTPBOOT
					#endif	// STATUSPAGE_REBOOT
					}
					break;	// Ende page_index 1
				#endif

				#if USE_SCHALTAUTOMATIK
				case(2):	// autom.htm

					switch(eth_buffer[a+1])
					{
						case ('A'):
							zeiten_speichern[0] = 1;
							break;
						case ('B'):
							zeiten_speichern[1] = 1;
							break;
					/*
						case ('c'):
							zeiten_speichern[2] = 1;
							break;
					*/
						case ('D'):
							zeiten_speichern[3] = 1;
							break;
					}

					automatikschalten = 1;
					break;	// Ende page_index 2
				#endif

				}

				//-----------------------------------------------------------
				// Schaltanweisung wurde gefunden
				//-----------------------------------------------------------

				http_entry[index].post_ptr = post_in;
			}

			//---------------------------------------------------------------
			// Submit schliet die Suche ab!
			//---------------------------------------------------------------

			if (eth_buffer[a] != *http_entry[index].post_ready_ptr++)
			{
				http_entry[index].post_ready_ptr = post_ready;
			}
			if(*http_entry[index].post_ready_ptr == 0) 
			{
				http_entry[index].post = 0;

				switch (page_index)
				{
				case(0):	// index.htm
					PORTC = PORT1_tmp;
					if(PORT1_write == 1)
					{
						eeprom_busy_wait();
						eeprom_write_byte((unsigned char*)DEFPC_EEPROM_STORE,PORTC);
						PORT1_write = 0;
					}
					if(PORT1_off > 0)
					{
						long_delay(var_array[12]);
						PORTC = PORT1_tmp - PORT1_off;
						PORT1_off = 0;
					}
					PORT1_tmp = 0;
					
					if(wochentag_speichern[0] == 1)
					{
						i2cout = i2cout+1;
						wochentag_speichern[0] = 0;
					}
					if(wochentag_speichern[1] == 1)
					{
						i2cout = i2cout+2;
						wochentag_speichern[1] = 0;
					}
					if(wochentag_speichern[2] == 1)
					{
						i2cout = i2cout+4;
						wochentag_speichern[2] = 0;
					}
					if(wochentag_speichern[3] == 1)
					{
						i2cout = i2cout+8;
						wochentag_speichern[3] = 0;
					}
					if(wochentag_speichern[4] == 1)
					{
						i2cout = i2cout+16;
						wochentag_speichern[4] = 0;
					}
					if(wochentag_speichern[5] == 1)
					{
						i2cout = i2cout+32;
						wochentag_speichern[5] = 0;
					}
					if(wochentag_speichern[6] == 1)
					{
						i2cout = i2cout+64;
						wochentag_speichern[6] = 0;
					}
					if(wochentag_speichern[7] == 1)
					{
						i2cout = i2cout+128;
						wochentag_speichern[7] = 0;
					}
					
					if (var_array[29] >0)
					{
						ret = i2c_start(var_array[31]+I2C_WRITE);
						if ( ret ) 
						{
						}else 
						{
							i2c_start_wait(var_array[31]+I2C_WRITE);     // set device address and write mode
							i2c_write(29+((var_array[29]-1)*6));        // erste Schreibadresse
							i2c_write(i2cout);   //										
							i2c_stop();		
						}
					}
					i2cout = 0;
					var_array[29]= 0;
					break;


				#if USE_SCHALTAUTOMATIK
				case(2):	// autom.htm
					var_array[10] &= ~(1 << 3);
					
					if(zeiten_speichern[0] == 1)
					{
						var_array[10] |= 1<<3;
						zeiten_speichern[0] = 0;
					}
					
					if(zeiten_speichern[1] == 1)
					{
						eeprom_busy_wait ();
						eeprom_write_word((unsigned int*)(SV_EEPROM_STORE),var_array[10]);
						zeiten_speichern[1] = 0;
					}
					if(zeiten_speichern[3] == 1)
					{
						for(uint8_t i=18;i<22;i++)  // Arrayposition 18 - 21
						{
							eeprom_busy_wait ();
							eeprom_write_word((unsigned int*)(SV_EEPROM_STORE + (2*(i-10))),var_array[i]);
						}
						zeiten_speichern[3] = 0;
					}
					
					break;
				#endif
				}
				break;

				//-----------------------------------------------------------
				// Submit gefunden
				//-----------------------------------------------------------
			}
		}
	}	

	// Wurde das Ende vom Header nicht erreicht
	// kommen noch weitere Stcke vom Header!
	if ((*http_entry[index].hdr_end_pointer != 0) || (http_entry[index].post == 1))
	{
		//Der Empfang wird Quitiert und es wird auf weiteres Headerstck gewartet
		tcp_entry[index].status =  ACK_FLAG;
		create_new_tcp_packet(0,index);
		//Warten auf weitere Headerpackete
		tcp_entry[index].app_status = 1;
		return;
	}	
	
	// Wurde das Passwort in den ganzen Headerpacketen gefunden?
	// Wenn nicht dann ausfhren und Passwort anfordern!
	if((!http_entry[index].http_auth) && tcp_entry[index].status&PSH_FLAG)
	{	
		//HTTP_AUTH_Header senden!
		http_entry[index].new_page_pointer = Page0;
		memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header2,(sizeof(http_header2)-1));
		tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
		create_new_tcp_packet((sizeof(http_header2)-1),index);
		tcp_entry[index].app_status = 2;
		return;
	}
	
	// Standart INDEX.HTM Seite wenn keine andere gefunden wurde
	if (!http_entry[index].new_page_pointer)
	{
		#if HTTP_VISITORS
		var_array[BESUCHER]++;	//Besucher Counter
		#endif
		
		http_entry[index].new_page_pointer = Page1;
		http_entry[index].http_header_type = TEXT_HTML;
	}	
	
	tcp_entry[index].app_status = 2;
	
	//Seiten Header wird gesendet
	switch(http_entry[index].http_header_type){
		case TEXT_PLAIN:
			memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header0,(sizeof(http_header0)-1));
			tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
			create_new_tcp_packet((sizeof(http_header0)-1),index);
		break;
		case TEXT_HTML:
			memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header1,(sizeof(http_header1)-1));
			tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
			create_new_tcp_packet((sizeof(http_header1)-1),index);
		break;
		case TEXT_CSS:
			memcpy_P((char*)&eth_buffer[TCP_DATA_START_VAR],http_header3,(sizeof(http_header3)-1));
			tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
			create_new_tcp_packet((sizeof(http_header3)-1),index);
		break;
	}
	return;
}

//----------------------------------------------------------------------------
//Daten Packete an Client schicken
void httpd_data_send (unsigned char index)
{	
	unsigned int a;
	unsigned char str_len;
	
	char var_conversion_buffer[CONVERSION_BUFFER_LEN];
	
	//Passwort wurde im Header nicht gefunden
	if(!http_entry[index].http_auth)
	{
		http_entry[index].new_page_pointer = Page0;
	}
	
	//kein Packet empfangen Retransmission des alten Packetes
	if (tcp_entry[index].status == 0) 
	{
		http_entry[index].new_page_pointer = http_entry[index].old_page_pointer;
	}
	http_entry[index].old_page_pointer = http_entry[index].new_page_pointer;

	for (a = 0;a<(MTU_SIZE-(TCP_DATA_START)-150);a++)
	{
		unsigned char b;
		b = pgm_read_byte(http_entry[index].new_page_pointer++);
		eth_buffer[TCP_DATA_START + a] = b;
		
		//Mssen Variablen ins Packet eingesetzt werden? ===> %VA@00 bis %VA@09
		if (b == '%')
		{
			//---------------------------------------------------------------
			// Variablen aus dem var_array einsetzen
			//---------------------------------------------------------------

			if (strncasecmp_P("VA@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	
				itoa (var_array[b],var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}

			if (strncasecmp_P("IX@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
																					//liest direkt vom I2C Bus
				   unsigned char ret;
				   unsigned int i2cwert;
					ret = i2c_start(var_array[31]+I2C_READ); 
					if ( ret ) {
					i2cwert = 0;
					
					}
					else {
					i2c_start_wait(var_array[31]+I2C_WRITE);
					i2c_write(b);
					i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
					i2cwert  = (i2c_readNak());               // read one byte and finish
					i2c_stop(); // Stop i2c
					}
				
				itoa (i2cwert,var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);
				
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}
			
			
			if (strncasecmp_P("IY@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
																					//liest direkt vom I2C Bus
				
				   unsigned char ret;
				   unsigned int i2cwert;
				   
					ret = i2c_start(var_array[31]+I2C_READ); 
					if ( ret ) {
					i2cwert = 0;
					
					}
					else {
					i2c_start_wait(var_array[31]+I2C_WRITE);
					i2c_write(b+95);
					i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
					i2cwert  = (i2c_readNak());               // read one byte and finish
					i2c_stop(); // Stop i2c
					}
				
				itoa (i2cwert,var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);
				
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}
			
			if (strncasecmp_P("IZ@",http_entry[index].new_page_pointer,3)==0)
			{	
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
																					//liest direkt vom I2C Bus
				
				   unsigned char ret;
				   unsigned int i2cwert;
					ret = i2c_start(var_array[31]+I2C_READ); 
					if ( ret ) {
					i2cwert = 0;
					
					}
					else {
					i2c_start_wait(var_array[31]+I2C_WRITE);
					i2c_write(b+191);
					i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
					i2cwert  = (i2c_readNak());               // read one byte and finish
					i2c_stop(); // Stop i2c
					}
				
				itoa (i2cwert,var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a = a + (str_len-1);
				
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}
			
			
			//---------------------------------------------------------------
            // Zeit in Webseite einfgen
			//---------------------------------------------------------------

            #if USE_NTP
            if (strncasecmp_P("TIME",http_entry[index].new_page_pointer,4)==0)
            {
                if (ntp_state == NTP_STATE_REQ_ERR)
                {
                    strcpy("dstr","-");
                }
                else
                {
                    ntp_struct dt;
                    decode_time(time, &dt);
                    sprintf_P(dstr, PSTR("%s, %s"), dt.datestr, dt.timestr);
                }
                str_len = strnlen(dstr,24);
                memmove(&eth_buffer[TCP_DATA_START+a],dstr,str_len);
                a += str_len - 1;
                http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+4;
            }
            #endif

			//---------------------------------------------------------------
			// IP-Adressen einsetzen
			//---------------------------------------------------------------

			#if HTTP_STATUSPAGE
			if (strncasecmp_P("IP",http_entry[index].new_page_pointer,2)==0)
            {
                b = 0;
                switch(pgm_read_byte(http_entry[index].new_page_pointer+2))
                {
                    case 'I':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),myip[0],myip[1],myip[2],myip[3]);
                        break;
                    case 'R':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),router_ip[0],router_ip[1],router_ip[2],router_ip[3]);
                        break;
                    case 'M':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),netmask[0],netmask[1],netmask[2],netmask[3]);
                        break;
					#if USE_DNS
                    case 'D':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),dns_server_ip[0],dns_server_ip[1],dns_server_ip[2],dns_server_ip[3]);
                        break;
					#endif
					#if USE_NTP
                    case 'N':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),ntp_server_ip[0],ntp_server_ip[1],ntp_server_ip[2],ntp_server_ip[3]);
                        break;
					#endif
					#if USE_SYSLOG
                    case 'S':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),syslog_server_ip[0],syslog_server_ip[1],syslog_server_ip[2],syslog_server_ip[3]);
                        break;
					#endif
					#if TFTP_CMD
                    case 'T':
						(*((unsigned long*)&tftp_server_ip[0])) = get_eeprom_value(TFTP_IP_EEPROM_STORE,IP(0,0,0,0));
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),tftp_server_ip[0],tftp_server_ip[1],tftp_server_ip[2],tftp_server_ip[3]);
                        break;
					#endif //TFTP_CMD
					#if USE_WOL
                    case 'W':
                        sprintf_P(dstr, PSTR("%i.%i.%i.%i"),wol_bcast_ip[0],wol_bcast_ip[1],wol_bcast_ip[2],wol_bcast_ip[3]);
                        break;
					#endif
                }
                str_len = strnlen(dstr,15);
                memmove(&eth_buffer[TCP_DATA_START+a],dstr,str_len);
                a += str_len-1;
                http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+3;
            }

			//---------------------------------------------------------------
			// MAC-Adressen einsetzen
			//---------------------------------------------------------------

			if (strncasecmp_P("MAC",http_entry[index].new_page_pointer,3)==0)
            {
                b = 0;
                switch(pgm_read_byte(http_entry[index].new_page_pointer+3))
				{
					case 'I':
						sprintf_P(dstr, PSTR("%02x:%02x:%02x:%02x:%02x:%02x"),mymac[0],mymac[1],mymac[2],mymac[3],mymac[4],mymac[5]);
						break;
					#if USE_WOL
					case 'W':
						sprintf_P(dstr, PSTR("%02x:%02x:%02x:%02x:%02x:%02x"),wol_mac[0],wol_mac[1],wol_mac[2],wol_mac[3],wol_mac[4],wol_mac[5]);
						break;
					#endif
				}
				str_len = strnlen(dstr,17);
				memmove(&eth_buffer[TCP_DATA_START+a],dstr,str_len);
				a += str_len - 1;
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+4;
            }
			#endif
			
			//---------------------------------------------------------------
			// Einsetzen des Port Status %PORTxy durch "checked" wenn Portx.Piny = 1
			// x: A..G  y: 0..7 
			//---------------------------------------------------------------

			if (strncasecmp_P("PORT",http_entry[index].new_page_pointer,4)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+5)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+4))
				{
					case 'A':
						b = (PORTA & (1<<pin));
						break;
					case 'B':
						b = (PORTB & (1<<pin));
						break;
					case 'C':
						b = (PORTC & (1<<pin));
						break;
					case 'D':
						b = (PORTD & (1<<pin));
						break; 
						
					case 'X':
						b = (var_array[10] & (1<<pin));
						break; 
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR(" checked"));
				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("\0"));
				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+6;
			}
			
			//---------------------------------------------------------------
			// Einsetzen des Pin Status %PIN@xy bis %PIN@xy durch "grn" oder "red" als CSS Klasse bergeben
			// x = 0 : PINA / x = 1 : PINB / x = 2 : PINC / x = 3 : PIND
			//---------------------------------------------------------------

			if (strncasecmp_P("PIN",http_entry[index].new_page_pointer,3)==0)
			{
				unsigned char pin  = (pgm_read_byte(http_entry[index].new_page_pointer+4)-48);	
				b = 0;
				switch(pgm_read_byte(http_entry[index].new_page_pointer+3))
				{
					case 'A':
						b = (PINA & (1<<pin));
						break;
					case 'B':
						b = (PINB & (1<<pin));
						break;
					case 'C':
						b = (PINC & (1<<pin));
						break;
					case 'D':
						b = (PIND & (1<<pin));
						break; 




					case 'a':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(29);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'b':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(35);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'c':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(41);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'd':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(47);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'e':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(53);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
					
						break;
					case 'f':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(59);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'g':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(65);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'h':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(71);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'i':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(77);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'j':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(83);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'k':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(89);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'l':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(95);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'm':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(101);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;
						
					case 'n':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(107);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'o':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(113);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'p':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(119);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'q':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(125);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'r':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(131);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 's':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(137);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 't':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(143);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						

						
					case 'u':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(149);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'v':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(155);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'w':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(161);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'x':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(167);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'y':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(173);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case 'z':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(179);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '1':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(185);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '2':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(191);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '3':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(197);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '4':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(203);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '5':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(209);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
						
					case '6':
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(215);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
					
					case '7':									//Dimmkurve 1
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(19);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
					
					
					case '8':									//Dimmkurve 2
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(20);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
					
					
					case '9':									//Dimmkurve 3
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(21);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
					
					
					case '0':									//Dimmkurve 4
						ret = i2c_start(var_array[31]+I2C_READ); 
						if ( ret ) {
						b = 0;
						}
					else 
					{
						i2c_start_wait(var_array[31]+I2C_WRITE); 
						i2c_write(22);                      // write address = b
						i2c_rep_start(var_array[31]+I2C_READ);       // set device address and read mode
						b  = (i2c_readNak() & (1<<pin));               // read one byte and finish
						i2c_stop(); // Stop i2c
					}
						break;						
					

	
				}
				
				if(b)
				{
					strcpy_P(var_conversion_buffer, PSTR("#FF0000")); // grn #008040 - grau #B0B0B0 -  rot #FF0000

				}
				else
				{
					strcpy_P(var_conversion_buffer, PSTR("#B0B0B0")); // rot #FF0000

				}
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;
				http_entry[index].new_page_pointer = http_entry[index].new_page_pointer+5;
			}

			//---------------------------------------------------------------
			// 1-Wire Temperatursensoren
			//---------------------------------------------------------------
			/*
			*       OW@nn   nn = 00 bis OW_MAXSENSORS-1 gibt Werte in 1/10 C aus
			*       OW@mm   mm = 20 bis OW_MAXSENSORS-1+20 gibt Werte in C mit einer Nachkommastelle aus
			*       d.h. OW@nn fr Balkenbreite verwenden und OW@mm fr Celsius-Anzeige
			*/

			#if USE_OW
			if (strncasecmp_P("OW@",http_entry[index].new_page_pointer,3)==0)
			{
				b = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				b +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);

				// Wert auslesen
				int16_t ow_temp = ow_array[b];
				str_len = 0;

				// evtl. Vorzeichen einfuegen:
				if ( ow_temp < 0 ) {
					ow_temp *= (-1);
					var_conversion_buffer[0] = '-';
					memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,1);
					a ++;
				}

				// Wert vor dem Komma einfgen
				itoa (ow_temp/10,var_conversion_buffer,10);
				str_len += strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);

				// Komma einfgen
				var_conversion_buffer[str_len] = ',';
				str_len++;
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len;

				// Wert nach dem Komma einfgen
				itoa (ow_temp%10,var_conversion_buffer,10);
				str_len = strnlen(var_conversion_buffer,CONVERSION_BUFFER_LEN);
				memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
				a += str_len-1;

				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}
			#endif  // USE_OW

			//---------------------------------------------------------------
			// TFTPFILE in Statusseite einfgen
			//---------------------------------------------------------------

			#if TFTP_CMD
			if (strncasecmp_P("TFTP",http_entry[index].new_page_pointer,4)==0)
			{
				// Variablen fr TFTP Fileausgabe
				char c;
				unsigned char tftpfile[16];
				uint16_t j = 0;

				// copy string from eeprom to memory
				do {
					c = eeprom_read_byte ((unsigned char*)TFTP_FILE_EEPROM_STORE+j);
					tftpfile[j] = c;
				} while ( (c != 0x00) && (c != 0xFF) && (j++ < 16) );

				sprintf_P(dstr, PSTR("%s"), tftpfile);
            
				str_len = strnlen(dstr,24);
				memmove(&eth_buffer[TCP_DATA_START+a],dstr,str_len);
				a += str_len - 1;
				http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+4;
			}
			#endif

			//---------------------------------------------------------------
			// Schaltzeit
			//---------------------------------------------------------------

			#if USE_SCHALTAUTOMATIK
			if (strncasecmp_P("TH@",http_entry[index].new_page_pointer,3)==0)
			{	
				
				inval = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				inval +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
				
				if (var_array[inval] >23)
				{
					var_array[inval] = 0;
				}
				if ( var_array[inval] < 10 ) 
				{
					var_conversion_buffer[0] = '0';
					var_conversion_buffer[1] = var_array[inval]+48;
					var_conversion_buffer[2] = 0;
				}
				else
				{
					itoa (var_array[inval],var_conversion_buffer,10);
				}
        		
               	str_len = strnlen(var_conversion_buffer,10);
        		memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
	       		a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;
			}
			
			if (strncasecmp_P("TM@",http_entry[index].new_page_pointer,3)==0)
			{	
				
				inval = (pgm_read_byte(http_entry[index].new_page_pointer+3)-48)*10;
				inval +=(pgm_read_byte(http_entry[index].new_page_pointer+4)-48);
				
				if (var_array[inval] > 59)
				{
					var_array[inval] = 0;
				}
				if ( var_array[inval] < 10) 
				{
					var_conversion_buffer[0] = '0';
					var_conversion_buffer[1] = var_array[inval]+48;
					var_conversion_buffer[2] = 0;
				}
				else
				{
					itoa (var_array[inval],var_conversion_buffer,10);
				}
				
               	str_len = strnlen(var_conversion_buffer,10);
        		memmove(&eth_buffer[TCP_DATA_START+a],var_conversion_buffer,str_len);
	       		a = a + (str_len-1);
       			http_entry[index].new_page_pointer=http_entry[index].new_page_pointer+5;	
			}			
			#endif	// USE_SCHALTAUTOMATIK

			//---------------------------------------------------------------
			// wurde das Ende des Packetes erreicht?
			// Verbindung TCP Port kann beim nchsten ACK geschlossen werden
			// Schleife wird abgebrochen keine Daten mehr!!

			if (strncasecmp_P("END",http_entry[index].new_page_pointer,3)==0)
			{	
				tcp_entry[index].app_status = 0xFFFD;
				break;
			}
		}
	}
	//Erzeugte Packet kann nun gesendet werden!
	tcp_entry[index].status =  ACK_FLAG | PSH_FLAG;
	create_new_tcp_packet(a,index);
	return;
}
#endif	// USE_HTTPD
