/*----------------------------------------------------------------------------
 Copyright:      Christoph Mai
 Author:         Christoph Mai
 Remarks:		 
 known Problems:
 Version:        28.10.2010
 Description:    ATtiny2313 i2c-Servocontroller

 This module interprets a stream of intel hex information and flashes it
 to the programm store.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation verffentlicht,
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
 (nach Ihrer Option) jeder spteren Version.

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben Sie an die Free Software Foundation,
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
----------------------------------------------------------------------------*/

#include "config.h"

#if TWI_SERVO

#include "i2c-servo.h"
#include "i2cmaster.h"

#include <avr/pgmspace.h> 
#include "usart.h"


//----------------------------------------------------------------------------
// Schreibe Servoposition
//----------------------------------------------------------------------------

void Servo_setposition(unsigned int servo, unsigned int position)
{
	uint8_t	servovar1,servovar2,servoval;

	servovar1 = (position >> 8 );
	servovar2 = position;

	I2CSERVO_DEBUG("servo %i: %i %2x%2x\r\n",servo,position,servovar2,servovar1);

	servoval = servo * 2;

	i2c_start_wait(DevI2CServo+I2C_WRITE);	// set device address & write mode
	i2c_write(servoval);					// write to servo
	i2c_write(servovar1);					// write variable to EEPROM
	i2c_write(servovar2);					// write variable to EEPROM
	i2c_stop();								// set stop conditon = release bus

}

//----------------------------------------------------------------------------
// Lese Servoposition
//----------------------------------------------------------------------------

uint16_t Servo_readposition(unsigned int servo)
{
	uint8_t	lowval,highval;
	uint8_t servoval = servo * 2;
	
	i2c_start_wait(DevI2CServo+I2C_WRITE);	// set device address & write mode
	i2c_write(servoval);					// servo
	i2c_rep_start(DevI2CServo+I2C_READ);	// set device address & read mode
	highval = i2c_readAck();				// read one highbyte from EEPROM
	lowval = i2c_readNak();					// read one lowbyte from EEPROM
	i2c_stop();
	
	I2CSERVO_DEBUG("servo %i: %2x%2x\r\n",servo,highval,lowval);
	
	uint16_t ret = ((highval << 8 ) | lowval); 
	return ret;
}

//----------------------------------------------------------------------------

#endif