/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        22.11.2007
 Description:    Programm zur Ansteuerung eines Standart LCD
				 (HD44870),(SED1278F) und kompatible

Addons :  LCD_4Bit wird zur Ansteuerung fr Parallel Ansteuerung benutzt (Ralf Figge)

------------------------------------------------------------------------------*/

#if USE_LCD_EXT
#ifndef _LCD_H_
	#define _LCD_H_
	
	//Prototypes
	extern void lcd_write (char,char);
#if LCD_4Bit
	extern char Read_LCD (char);
#else
	void shift_data_out (char);
#endif //LCD_4Bit

	extern void lcd_init (void);
	extern void lcd_clear (void);
	extern void lcd_print_P (unsigned char,unsigned char,const char *Buffer,...);
	extern void lcd_print_str (char *Buffer);
	
	#define lcd_print(a,b,format, args...)  lcd_print_P(a,b,PSTR(format) , ## args)
	
#if LCD_4Bit
	//Im 4Bit Mode LCD_D4-->PORTx.0 ........ LCD_D7-->PORTx.3
	//LCD_RS --> PORTx.4 | LCD_RW --> PORTx.5 | LCD_E --> PORTx.6 | PORTx.7-->NotConnect
	
	#define LCD_Port_DDR		DDRD	//Ext. Port am AVR-NET-IO
	#define LCD_Port_Write		PORTD
	#define LCD_Port_Read		PIND
	#define LCD_Port2_DDR		DDRB	//Ext. Port am AVR-NET-IO
	#define LCD_Port2_Write		PORTB
	#define LCD_Port2_Read		PINB
		
	#if LCD_4Bit_POLLIN					//Ext. Port am AVR-NET-IO und Pollin Belegung
		#define LCD_RS			3 		//Pin fr RS
		#define LCD_RW			2		//Pin fr Read/Write
	#else
		#define LCD_RS			2 		//Pin fr RS
		#define LCD_RW			3		//Pin fr Read/Write
	#endif
		
	#define LCD_E				0 		//Pin fr Enable (PORTB.0)
	#if LCD_BACKLIGHT
		#define LCD_LIGHT_PIN	3		//Pin fr Backlight (PORTB.3)
	#endif

	#define LCD_DataOutput		0xF0
	#define LCD_DataInput		0x00

	#define BUSYBIT					7
#else
	#define PORT_LCD_DATA_ENABLE	PORTD
	#define DDR_LCD_DATA_ENABLE		DDRD
	#define LCD_DATA_ENABLE			5		// Pin 6 am LCD
	
	#define PORT_LCD_CLOCK			PORTD
	#define DDR_LCD_CLOCK			DDRD
	#define LCD_CLOCK				4		// Pin 8 am 74HC164

	#define PORT_LCD_DATA			PORTD
	#define DDR_LCD_DATA			DDRD
	#define LCD_DATA				6		// Pin 1+2 am 74HC164
	
	#define LCD_RS_PIN 				0		// Pin fr RS 			Pin  3 am 74HC164 zum LCD
	#define LCD_LIGHT_PIN			7		// Pin fr Backlight	Pin 13 am 74HC164 zum LCD
#endif //LCD_4Bit

	#define BUSY_WAIT				6000
	
	#define NOP()	asm("nop")
	#define WAIT(x) for (unsigned long count=0;count<x;count++){NOP();}

#endif //_LCD_H_
#endif //USE_LCD_EXT
