/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        angepasst von Shortie
 known Problems: none
 Version:        24.01.2011
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "cmd.h"

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#if USE_ADC
	#include "analog.h"
#endif

#if USE_HTTPD
	#include "httpd.h"
	#include "base64.h"
#endif

#if USE_WOL
	#include "wol.h"
#endif

#if USE_TELNETD
	#include "telnetd.h"
#endif

#if USE_NTP
	#include "ntp.h"
#endif

#if USE_LCD_EXT
	#include "lcd.h"
#endif

#if USE_MAIL
	#include "base64.h"
	#include "sendmail.h"
#endif

#if USE_DHCP
	#include "dhcpc.h"
#endif

#if USE_DNS
	#include "dnsc.h"
#endif

#if USE_SYSLOG
	#include "syslog/syslog.h"
#endif

#if USE_UDPSEND
	#include "udp_send/udp_send.h"
#endif

#if USE_OW
	#include "1-wire/ds18x20.h"
	#include "1-wire/messung.h"
	
	uint8_t ow_auslesen	= 0;	// 0 -> Sensoren nicht auslesen

	#if OW_MINMAX
	uint8_t minmax		= 1;	// 1 -> Zurcksetzen der Min/Max-Werte 1x/Tag
	#endif //OW_MINMAX
#endif

#if (LCD_BACKLIGHT && LCD_BL_AUTO)
	uint8_t bl_automatik = 0;	// 0 -> Backlight Automatik aus
#endif

#if USE_TWI
	#include "i2c/i2cmaster.h"
#endif

#if ADDON_IR_RC5
	#include "infrared/rc5.h"
#endif

#if ADDON_LCD
	#include "i2clcd/i2clcd.h"
#endif

#if ADDON_LEDS
	#include "led/led.h"
#endif

#if ADDON_SDCARD
	#include "sdcard/fat16.h"
	#include "sdcard/sdcard.h"
#endif

#if SDCARD_LOGFILE
	#include <stdbool.h>
	#include "sdcard/sd_logging.h"
#endif

#if (USE_LCD_EXT || ADDON_LCD)
	char zeile[20];
	unsigned char lcd_update = 0;
#endif

#if USE_SCHALTAUTOMATIK
	int ende;
#endif

#if USE_HTTP_CLIENT
	#include "http_client/http_client.h"
#endif

//----------------------------------------------------------------------------
// debounce Funktion um Taster zu entprellen
//----------------------------------------------------------------------------

#define debounce( port, pin ) 	/* Backslashes am Ende der Zeilen mssen */	\
({								/* sein und drfen nicht entfernt werden */ \
	static uint8_t flag = 0;	/* new variable on every macro usage */		\
	uint8_t deb_i = 0;														\
																			\
	if( flag ){							/* check for key release: */		\
		for(;;){						/* loop ... */						\
			if( !(port & 1<<pin) ){	/* ... until key pressed or ... */	\
				deb_i = 0;				/* 0 = bounce */					\
				break;														\
			}																\
			_delay_us( 98 );			/* * 256 = 25ms */					\
			if( --deb_i == 0 ){		/* ... until key >25ms released */	\
				flag = 0;				/* clear press flag */				\
				deb_i = 0;				/* 0 = key release debounced */		\
				break;														\
			}																\
		}																	\
	} else {							/* else check for key press: */		\
		for(;;){						/* loop ... */						\
			if( (port & 1<<pin) ){		/* ... until key released or ... */	\
				deb_i = 0;				/* 0 = bounce */					\
				break;														\
			}																\
			_delay_us( 98 );			/* * 256 = 25ms */					\
			if( --deb_i == 0 ){		/* ... until key >25ms pressed */	\
				flag = 1;				/* set press flag */				\
				deb_i = 1;				/* 1 = key press debounced */		\
				break;														\
			}																\
		}																	\
	}																		\
	deb_i;								/* return value of Macro */			\
})


//----------------------------------------------------------------------------
// Variablen-Array / Variablen-Array2 bei USE_UDPSEND
//----------------------------------------------------------------------------
// zum Abspeichern verschiedener Werte und zum Einfgen in die Webseite 
// als %VA@00 bis %VA@xx
//
// VA0-7	-> PORT A0-7
// VA8		-> PORT C
// VA9		-> Port D
// VA10		-> Sonderfunktionen (siehe README.txt)
// VA11		-> Tasterdefinition PORT C
// VA12		-> Tastdauer von VA11 in Millisekunden
// VA13		-> Intervall in Sekunden fr Logfile schreiben - Default: 300 Sekunden
// VA14-15	-> Einschaltzeit 1 Automatik (hh mm)
// VA16-17	-> Ausschaltzeit 1 Automatik (hh mm)
// VA18-19	-> Einschaltzeit 2 Automatik (hh mm)
// VA20-21	-> Ausschaltzeit 2 Automatik (hh mm)
// VA22-26	-> Reserve (5 Werte)
// VA27		-> Counter Webseite
// VA28		-> Dimmer Nr.
// VA29		-> Auswahl letzte Nr.
// VA30		-> 
// VA31		-> I2C Adresse

unsigned int var_array[MAX_VAR_ARRAY] = {0,0,0,0,0,0,0,0,0,0, 15, 0,500, 300,0,0,0,0,0,0,0,0,0,0,0,0,0, 0};
#if USE_UDPSEND
unsigned int va2_array[MAX_VAR_ARRAY] = {0,0,0,0,0,0,0,0,0,0,  0, 0,  0,   0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0};
#endif	//USE_UDPSEND

//----------------------------------------------------------------------------
// Hier startet das Hauptprogramm
//----------------------------------------------------------------------------

int main(void)
{  	
    unsigned long a;
	unsigned char oldss = 0;
	unsigned char oldmm = 0;
	unsigned char oldhh = 0;
	
	// Konfiguration der Ausgnge bzw. Eingnge
	// Definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;

	#if ADDON_SDCARD
	DDRB |= (1 << PB3);	// PortB Pin3 Chip Select fr SD-Karte
	PORTB &= ~(1 << PB3);	// select_card
	PORTB |= (1 << PB3);	// unselect_card
	#endif

	PORTC = 0;
	
	// Pullups einschalten PortA
	PORTA |= (1 << PORTA0) | (1 << PORTA1) | (1 << PORTA2) | (1 << PORTA3);
	
	#if USE_ADC
		ADC_Init();
	#else
		PORTA |= (1 << PORTA4) | (1 << PORTA5) | (1 << PORTA6);
		#if !OW_PORTA
		PORTA |= (1 << PORTA7);
		#endif
	#endif
	
	DDRB  &= ~(1 << DDB1); 	// Pin B1 (Pollin Prog.-Jumper) als Eingang definieren
	PORTB |= (1 << PORTB1);	// Pullup einschalten PortB1
	
	eepvar_init();				// Grundeinstellungen einlesen

    usart_init(BAUDRATE); 		// setup the UART
	
	usart_write("\r\nSystem Ready\r\n");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n\r\n");

	for(a=0;a<1000000;a++) { asm("nop"); };

	//Applikationen starten
	stack_init();
	
	#if USE_HTTPD
	httpd_init();
	#endif
	
	#if USE_TELNETD
	telnetd_init();
	#endif
	
	//LCD_BACKLIGHT aktivieren
	#if LCD_BACKLIGHT
		lcd_backlight = 1;
	#endif

	//LCD initialisieren
	#if USE_LCD_EXT
		lcd_init();
		lcd_clear();
		
		//Spielerrei mit einem LCD
		lcd_print(0,1,"System Booting");
		#if LCD_USER_CHARS
		lcd_print(1,3,"I %c my AVR",3);
		#endif
	#endif

	#if	USE_TWI
        i2c_init();
	#endif
	
	#if ADDON_LCD
        i2clcd_init();
		i2clcd_command(LCD_DISPLAYON | LCD_CURSOROFF);
		
		i2clcd_printlr(1,1," System Booting ");
		#if LCD_USER_CHARS
		sprintf(zeile, "   I %c my AVR   ",0x03);
		i2clcd_printlr(2,1,zeile);
		#endif
	#endif

	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 
	
    #if USE_DHCP
    dhcp_init();
    if ( dhcp() == 0) {
		save_ip_addresses();
		dhcp_ok = 1;
    } else {
		usart_write("DHCP fail\r\n\r\n");
		read_ip_addresses(); //get from EEPROM
		dhcp_ok = 0;
    }
    #endif //USE_DHCP
	
    usart_write("IP   %1i.%1i.%1i.%1i"    , myip[0]     , myip[1]     , myip[2]     , myip[3]);
	#if USE_DHCP
	if(dhcp_ok == 1) {
		usart_write(" (DHCP)");
	}
	#endif //USE_DHCP
	usart_write("\r\n");
    usart_write("MASK %1i.%1i.%1i.%1i\r\n", netmask[0]  , netmask[1]  , netmask[2]  , netmask[3]);
    usart_write("GW   %1i.%1i.%1i.%1i\r\n", router_ip[0], router_ip[1], router_ip[2], router_ip[3]);

    #if USE_DNS
    usart_write("DNS  %1i.%1i.%1i.%1i\r\n", dns_server_ip[0], dns_server_ip[1], dns_server_ip[2], dns_server_ip[3]);
    #endif //USE_DNS
    
	#if USE_NTP
		#if !USE_DHCP
        #if USE_DNS
        dns_init();
		if ( dns_resolve("1.de.pool.ntp.org") == 0) {	// resolve NTP server
			for (unsigned char count = 0; count<4; count++) {
				eeprom_busy_wait ();
				eeprom_write_byte((unsigned char *)(NTP_IP_EEPROM_STORE + count),dns_resolved_ip[count]);
			}
		} else {
			usart_write("DNS Err.\r\n");
		}
        #endif	// USE_DNS
		#endif	// !USE_DHCP
    
	ntp_init(); 
   
	usart_write("NTP  %1i.%1i.%1i.%1i\r\n", ntp_server_ip[0], ntp_server_ip[1], ntp_server_ip[2], ntp_server_ip[3]);
   
	for(a=0;a<1000000;a++) { asm("nop"); };
   
	ntp_request();
   
    if (ntp() != 0) {
		usart_write("NTP Err.\r\n");
	} else {
		command_time();
	}
	#endif	// USE_NTP

	#if USE_WOL
        wol_init();
	#endif	// USE_WOL
    
    #if USE_MAIL
        mail_client_init();
	#endif	// USE_MAIL  

	#if USE_OW
		uint8_t i;
		for (i=0;i<OW_MAXSENSORS;i++) {
			ow_array[i]=OW_START;
		}
		#if OW_MINMAX
		for (i=OW_MAXSENSORS;i<OW_MAXSENSORS*3;i++) {
			ow_array[i]=OW_MINMAX_START;
		}
		#endif // OW_MINMAX
		messung_init();
	#endif	// USE_OW

	#if ADDON_SDCARD
		f16_init();
	#endif

	#if ADDON_IR_RC5
		rc5_init();
	#endif

	#if USE_UDPSEND
		udp_send_init();
	#endif

	#if USE_SYSLOG
		usart_write("SYSLOG  %1i.%1i.%1i.%1i\r\n", syslog_server_ip[0], syslog_server_ip[1], syslog_server_ip[2], syslog_server_ip[3]);
		syslog_send(5,0,"system: AVR-NET-IO is running");
	#endif	// USE_SYSLOG
	
	#if USE_LCD_EXT
		lcd_clear();
		lcd_print(0,0,"System ready   ");
	#endif	// USE_LCD_EXT

	#if ADDON_LCD
		i2clcd_command(LCD_CLEAR);
		i2clcd_printlr(1,1,"System ready   ");		// Print a string
	#endif	// ADDON_LCD

	#if (LCD_BACKLIGHT && LCD_BL_AUTO)
		lcd_bl_timer = LCD_BL_TIME;
	#endif	// LCD_BL_AUTO

	#if ADDON_LEDS
		LED_on(1);
		#if LED3_PWM
		InitPWM_LED3();
		#endif	// LED3_PWM
	#endif		// ADDON_LEDS
	
	#if SDCARD_LOGFILE
		machineStatus.LogEnabled = (var_array[10] & 0x04); // Bit 3 von var_array[10] auslesen
	
		if (machineStatus.LogEnabled) {
			machineStatus.LogInit = true;
		}
	#endif	// SDCARD_LOGFILE

	#if USE_SCHALTAUTOMATIK
	automatikschalten = 1;	// automatische Schaltvorgnge ausfhren bei Start
	#endif	// USE_SCHALTAUTOMATIK
	var_array[31]=adresse1;		//Voreinstellung Dimmer Adresse I2C
	var_array[28]=1;			//Voreinstellung Dimmer Nr.
//---------------------------------------------------------------------------
// Hauptschleife
//---------------------------------------------------------------------------

	while(1)
	{
		//-------------------------------------------------------------------
		// Zeit aktualisieren
		//-------------------------------------------------------------------
		
		unsigned char hh = (time/3600)%24;
		unsigned char mm = (time/60)%60;
		unsigned char ss = time %60;
		unsigned char ret;			//fr Abfrage i2c bereit
		unsigned int i2cwert = 0;
		//-------------------------------------------------------------------
		// Zeitabhngige Dinge erledigen
		//-------------------------------------------------------------------

		if(ss != oldss) {			// bei Sekundenwechsel
		
			#if ADDON_LEDS
			LED_toggle(2);			// LED_2 wechselt im Sekundentakt
			#endif
			
			#if LED3_PWM
			SetPWMOutput(ss*4);		// PWM-Demo an LED3
			#endif

			if(ss == 0) {			// nur bei Minutenwechsel
				#if SDCARD_LOGFILE
				if((hh == 0) && (mm == 0) && (ss == 0) && machineStatus.LogEnabled) {
					// Um 0.00 Uhr wird ein neues Logfile angelegt
					machineStatus.LogInit = true;
				}
				#endif

				#if USE_SCHALTAUTOMATIK
				automatikschalten = 1;	// automatische Schaltvorgnge ausfhren
				#endif
				
				#if USE_HTTP_CLIENT
				HTTP_CLIENT_state = 1;	// Daten ber HTTP-GET an WEbserver senden
				#endif
				
				zeith = hh;
				zeitm = mm;	
/*----------------------------------------------------------------------------
				
			if((hh > 0) && (mm > 0)) { 						//NTP muss erst mal die Zeit lesen		
					ret = i2c_start(adresse1+I2C_READ); 		//braucht der Dimmer eine neue Zeit?
					if ( ret ) {
					}
					else {
					i2c_start_wait(adresse1+I2C_WRITE);
					i2c_write(30);
					i2c_rep_start(adresse1+I2C_READ);       // set device address and read mode
					i2cwert  = (i2c_readNak());               // read one byte and finish
					i2c_stop(); // Stop i2c
					}	
					if(i2cwert > 127) {
						ret = i2c_start(adresse1+I2C_WRITE);		//Zeit zum Dimmer
						if ( ret ) 
						{
						}else 
						{
						if (zeitw == 0) {zeitw = 7;}			//Sonntag soll 7 sein und nicht 0
							i2c_start_wait(adresse1+I2C_WRITE);     // set device address and write mode
							i2c_write(23);                      // erste Schreibadresse
							i2c_write(zeitm);   // Minute
							i2c_write(zeith);   // Stunde
							i2c_write(zeitt);   // Tag
							i2c_write(zeitd);   //	Monat
							i2c_write(zeitw);	// Wochentag
							i2c_write(0);		// Platzhalter
							i2c_write(0);		// Platzhalter
							i2c_write(0);		// Flag auf 0
							i2c_stop();		
						}
					}
				}
				
----------------------------------------------------------------------------*/				
				
				
			}
			oldss = ss;


			if(((hh == 0) && (mm == 0) && (hh != oldhh))|(zeitt == 0 )) { 
				// Um 0.00 Uhr oder nach Neustart wird das Datum gendert
				ntp_struct dt;
				decode_time(time, &dt);				//Neues Datum auslesen
				zeitt = dt.day;						// Altueller Tag
				zeitd = dt.month;					// Altueller Monat
				}
			
			oldss = ss;
			oldmm = mm;
			oldhh = hh;




			
			#if (USE_LCD_EXT || ADDON_LCD)
			lcd_update = 1;
			#endif
		}

		//-------------------------------------------------------------------
		// Schaltautomatik
		//-------------------------------------------------------------------
		
		#if USE_SCHALTAUTOMATIK

		// Bit 4 von var_array[10] und automatikschalten auswerten
		if((var_array[10] & 0x08) && (automatikschalten == 1))	 {
		/*
			//---------------------------------------------------------------
			// Automatik 1 -> Relais 7
			//---------------------------------------------------------------
			
			ende  = ((var_array[16]*60) + var_array[17]);
			if(ende < ((var_array[14]*60) + var_array[15]))
			{
				ende = ende+1440;
			}
			if (((var_array[14]*60 + var_array[15]) <= (hh*60 + mm)) && ((hh*60 + mm) < ende))
			{
				PORTC |= (1<<PC6);		// Relais 7 ein
			} else {
				PORTC &= ~(1<<PC6);		// Relais 7 aus
			}
		*/
		
			//---------------------------------------------------------------
			// Automatik 2 -> Relais 8
			//---------------------------------------------------------------

			ende  = ((var_array[20]*60) + var_array[21]);
			if(ende < ((var_array[18]*60) + var_array[19])) {
				ende = ende+1440;

			}
			if (((var_array[18]*60 + var_array[19]) <= (hh*60 + mm)) && ((hh*60 + mm) < ende)) {
				#if LCD_BACKLIGHT
				lcd_backlight = 1;		// LCD Hintergrundbeleuchtung ein
				#endif
				PORTC |= (1<<PC7);		// Relais 8 ein
			} else {
				#if LCD_BACKLIGHT
				lcd_backlight = 0;		// LCD Hintergrundbeleuchtung aus
				#endif
				PORTC &= ~(1<<PC7);	// Relais 8 aus
			}

			//---------------------------------------------------------------
			automatikschalten = 0;		// automatikschalten wurde ausgefhrt
		}
		#endif

		//-------------------------------------------------------------------
		// Variable mit Portwerten fllen
		//-------------------------------------------------------------------

		var_array[0] = (PINA&0b00000001);
		var_array[1] = (PINA&0b00000010)>>1;
		var_array[2] = (PINA&0b00000100)>>2;
		var_array[3] = (PINA&0b00001000)>>3;
		
		// ADC-Werte werden in analog.c in die Variablen geschrieben
		// hier werden bei USE_ADC 0 die Werte geschrieben.
		#if !USE_ADC
		var_array[4] = (PINA&0b00010000)>>4;
		var_array[5] = (PINA&0b00100000)>>5;
		var_array[6] = (PINA&0b01000000)>>6;
		#if !OW_PORTA
		var_array[7] = (PINA&0b10000000)>>7;
		#endif	// !OW_PORTA
		#endif	// !USE_ADC
		
		var_array[8] = PORTC;

		//-------------------------------------------------------------------

		#if USE_ADC
		ANALOG_ON;
		#endif
		
	    eth_get_data();
		
        //Terminalcommandos auswerten
		if (usart_status.usart_ready) {
            usart_write("\r\n");
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}
		
        //Empfang von Zeitinformationen
		#if USE_NTP
		if(!ntp_resync) {
			ntp_request();
			ntp_resync = NTP_REFRESH;
		}
		#endif //USE_NTP
	
        //Versand von E-Mails
        #if USE_MAIL
        if (mail_enable == 1) {
            mail_send();
            mail_enable = 0;
        }
        #endif //USE_MAIL
        
        //Rechner im Netzwerk aufwecken
        #if USE_WOL
		if (wol_enable == 1) {
			wol_request();
			wol_enable = 0;
		}
        #endif //USE_WOL
        
        #if USE_DHCP
		if(dhcp_ok == 1) {			// nur ausfuehren wenn DHCP beim Start verfgbar war
			if ( dhcp() != 0) {	//check for lease timeout
				usart_write("dhcp lease renewal failed\r\n");
				RESET();
			}
		}
		#endif //USE_DHCP
  
		#if USE_TELNETD
		telnetd_send_data();	// USART Daten fr Telnetanwendung?
		#endif
        
		#if USE_NETCMDS
        if(ping.result) {
            usart_write("Get PONG: %i.%i.%i.%i\r\n",ping.ip1[0],ping.ip1[1],ping.ip1[2],ping.ip1[3]); 
            ping.result = 0;
        }
		#endif

		#if (UDPSEND_SPORT!=0)
		if(!udp_send_timer) {
			udp_send(UDPSEND_IP);
			udp_send_timer = UDPSEND_REFRESH;
		}
		#endif //UDPSEND_SPORT

		#if (LCD_BACKLIGHT && LCD_BL_AUTO)
		bl_automatik = (var_array[10] & 0x02);	// Bit 2 von var_array[10] auslesen
		
		if (bl_automatik == 1) {
			if((!lcd_bl_timer) && (lcd_backlight))	{ // Wenn lcd_bl_timer = 0 und lcd_backlight = 1
				#if LCD_4Bit
				LCD_Port2_Write &= ~(1<<LCD_LIGHT_PIN);	// LCD-Backlight ausschalten
				#endif
				lcd_backlight = 0;
			}
		}
		#endif	//LCD_BL_AUTO
		
		//-------------------------------------------------------------------
		// Infrarotempfnger auslesen und RC5 Code interpretieren
		//-------------------------------------------------------------------
		
		#if ADDON_IR_RC5
		cli();
		int fbc = rc5_data;	// read two bytes from interrupt !
		rc5_data = 0;
		sei();
		
		if(fbc) {
			if (fbc != lastRC5data) {
				// neuer Tastendruck
				lastRC5data = fbc;	// letzten Tastendruck speichern

				rc5_device = (fbc >> 6 & 0x1F);
				rc5_keycode = (fbc & 0x3F) | (~fbc >> 7 & 0x40);

				RC5_DEBUG("\r\nRC5 %i",( fbc >> 11 & 1));	// Toggle Bit
				RC5_DEBUG(" %i", rc5_device);				// Device address
				RC5_DEBUG(" %i -",rc5_keycode);				// Key Code
				
				// RC5_DEBUG kann in infrared/rc5.h aktiviert werden um
				// die Device address und (weitere) Tastencodes einer 
				// RC5-Fernbedienung herauszufinden.
				
				//-----------------------------------------------------------
				// Schaltvorgnge ber Fernbedienung
				//-----------------------------------------------------------

				if (rc5_device == RC5DEV_ID)		// Nur Codes fr angebene Device Adresse verarbeiten
				{									// Wenn mehrere Gerte RC5 Codes benutzen
					if (rc5_keycode == RC5KEY_0) {	// Wenn FB-Taste 0 gedrckt
						PORTC = 0x00;				// schalte alle Ausgnge an PORTC aus
					}

					if (( rc5_keycode >= RC5KEY_1 ) && (rc5_keycode <= RC5KEY_8)) { // Wenn FB-Taste 1 - 8
						PORTC = PINC ^ (1 << (rc5_keycode - 1));					 // schalte Ausgang PC0 - PC7
					}

					if (rc5_keycode == RC5KEY_9) {	// Wenn FB-Taste 9 gedrckt
						#if USE_TWI
						PORTC = 0xFC;				// schalte alle Ausgnge ein auer TWI/i2c
						#else
						PORTC = 0xFF;				// schalte alle Ausgnge ein
						#endif
					}

					#if LCD_BACKLIGHT
					if (rc5_keycode == RC5KEY_VUP)	{	// FB-Taste VOL+
						lcd_backlight = 1;
						lcd_update = 1;
						#if LCD_BL_AUTO
						lcd_bl_timer = LCD_BL_TIME;
						#endif	//LCD_BL_AUTO
					}
					if (rc5_keycode == RC5KEY_VDN)	{	// FB-Taste VOL-
						lcd_backlight = 0;
						lcd_update = 1;
					}
					#endif
				}
			} else {
				RC5_DEBUG("\r\nTaste %i wird gehalten",rc5_keycode);
			}
		}
		#endif

		//-------------------------------------------------------------
		// Taster an PB1 abfragen und LCD-Backlight schalten
		//-------------------------------------------------------------

		#if LCD_BACKLIGHT
		if (debounce(PINB, PB1)) {	// Falls Taster an PIN PB1 gedrueckt..
			#if LCD_4Bit
			LCD_Port2_Write = LCD_Port2_Read ^ ( 1 << LCD_LIGHT_PIN );	// LCD-Backlight an- bzw. ausschalten
			#endif
			
			if(lcd_backlight == 1) {
				lcd_backlight = 0;
			} else {
				lcd_backlight = 1;
			}
			
			lcd_update = 1;
			
			#if LCD_BL_AUTO
			lcd_bl_timer = LCD_BL_TIME;
			#endif	//LCD_BL_AUTO
		}
		#endif	//LCD_BACKLIGHT

		//-----------------------------------------------------
		// 1-Wire Bus abfragen und Temperaturen auslesen
		//-----------------------------------------------------
		
		#if USE_OW
		
		ow_auslesen = (var_array[10] & 0x01);	// Bit 1 von var_array[10] auslesen
		
		if ((ss%10 == 8) && (ow_auslesen != 0)) {
			#if OW_EXT_POWER
			start_OW();
			#endif
			lese_Temperatur();	// Temperatursensoren lesen

			#if OW_MINMAX
			for (i=0;i<OW_MAXSENSORS;i++) {
				if ((( hh == 00 )&&( mm == 00 )) || (minmax == 1)) {
					// Zurcksetzen der Min/Max_Werte 1x/Tag
					// auf die gerade aktuellen Temperaturen
					ow_array[i+OW_MAXSENSORS] = ow_array[i];
					ow_array[i+OW_MAXSENSORS*2] = ow_array[i];
				} else {
					// Abgleich der Temp. mit den gespeicherten Min/Max-Werten
					if (ow_array[i]  < ow_array[i+OW_MAXSENSORS])
						ow_array[i+OW_MAXSENSORS] = ow_array[i];
					if (ow_array[i]  > ow_array[i+OW_MAXSENSORS*2])
						ow_array[i+OW_MAXSENSORS*2] = ow_array[i];
				}
			}
			minmax = 0;
			#endif //OW_MINMAX
		}
		#endif	//USE_OW

		//-------------------------------------------------------------
		// SD-Card auf ADD-ON Board und Logging auf SD-Card
		//-------------------------------------------------------------
		
		#if ADDON_SDCARD
		f16_check();    // Ein-/Ausstecken der SD-Karte erkennen
		#endif

		#if SDCARD_LOGFILE
		machineStatus.LogEnabled = (var_array[10] & 0x04); // Bit 3 von var_array[10] auslesen
					
		if(machineStatus.LogEnabled) {		// Wenn Logging auf SD-Karte eingeschalten 
		
			if(!sdcard_log_timer) {						// Bei Ablauf des SD-Card Timers  ...
				machineStatus.LogSchreiben = true;		// ... LogSchreiben zulassen
				
				// Wenn Einstellung geringer als 10 Sekunden auf Defaultwert (300 Sek.) setzen.
				if(var_array[13] < 10) {
					var_array[13] = 300;
				}
				sdcard_log_timer = var_array[13];
			}
		
			if (machineStatus.LogInit) {
				log_init();						// neue Logdatei anlegen
				machineStatus.LogInit = false;
			}
		
			if (machineStatus.LogSchreiben) {
				log_status();					// Logfile schreiben
				machineStatus.LogSchreiben = false;
			}
		}
		#endif

		//-------------------------------------------------------------
		// Daten ber HTTP-GET an Webserver senden
		//-------------------------------------------------------------
		
		#if USE_HTTP_CLIENT
		if(HTTP_CLIENT_state) http_client_do();
		#endif	// USE_HTTP_CLIENT
		
//---------------------------------------------------------------------
// ab hier eigenen Code wie Ausgaben oder weitere Verarbeitung der 
// Werte einfgen
//---------------------------------------------------------------------

		#if(USE_LCD_EXT || ADDON_LCD)
		if(lcd_update) {
		#if USE_LCD_EXT
			#if USE_OW
			lcd_print(0,0,"Aussen ");
			dtostrf(ow_array[0] / 10.0,5,1,zeile);
			lcd_print_str(zeile);
			lcd_print_str("C");
			lcd_print(1,0,"Innen  ");
			dtostrf(ow_array[1] / 10.0,5,1,zeile);
			lcd_print_str(zeile);
			lcd_print_str("C");

			lcd_print(2,0,"%2i:%2i:%2i",hh,mm,ss);
			#else
			lcd_print(0,0,"%2i:%2i:%2i    ",hh,mm,ss);
			#endif	//USE_OW
			
			#if LCD_USER_CHARS
/*
			// Demo der 8 benutzerdefinierten Zeichen
			uint8_t udc;
			for (udc=0;udc<8;udc++) {
				lcd_print(3,udc,"%c",udc);	// Die 8 benutzerdefinierten Zeichen
			}								// in Zeile 4 des Displays anzeigen
*/
/*
			// Status von PortC in Zeile 4 anzeigen
			for (uint8_t udc=0;udc<8;udc++) {
				if(PINC & (1<<(udc))) lcd_print(3,udc,"%c",6);
				else lcd_print(3,udc,"%c",7);
			}
*/
			#endif	// LCD_USER_CHARS
		#endif	//USE_LCD_EXT
		
		#if ADDON_LCD
			#if USE_OW
			i2clcd_printlr(1,1,"Aussen ");
			dtostrf(ow_array[0] / 10.0,5,1,zeile);
			i2clcd_print(zeile);
			i2clcd_print("C");
			i2clcd_printlr(2,1,"Innen  ");
			dtostrf(ow_array[1] / 10.0,5,1,zeile);
			i2clcd_print(zeile);
			i2clcd_print("C");
			
			sprintf(zeile, "%02d:%02d:%02d", hh, mm, ss);
			i2clcd_printlr(3,1,zeile);
			#else
			sprintf(zeile, "%02d:%02d:%02d", hh, mm, ss);
			i2clcd_printlr(1,1,zeile);
			#endif	// USE_OW
			
			#if LCD_USER_CHARS	
/*
			// Demo der 8 benutzerdefinierten Zeichen
			uint8_t udc;
			for (udc=0;udc<8;udc++) {
				i2clcd_putcharlr(4,udc+1,udc);	// Die 8 benutzerdefinierten Zeichen
			}									// in Zeile 4 des Displays anzeigen
*/
/*
			// Status von PortC in Zeile 4 anzeigen
			i2clcd_printlr(4,1,"--");
			for (uint8_t udc=2;udc<8;udc++) {
				if(PINC & (1<<(udc))) i2clcd_putcharlr(4,udc+1,6);
				else i2clcd_putcharlr(4,udc+1,7);
			}
*/
			#endif	// LCD_USER_CHARS
		#endif	// ADDON_LCD

			lcd_update = 0;
		}
		#endif	// if(USE_LCD_EXT || ADDON_LCD)
   }	//while (1)
//---------------------------------------------------------------------
	return(0);
}
