/*----------------------------------------------------------------------------
 Copyright:      Christoph Mai
 Author:         Christoph Mai
 Remarks:		 
 known Problems: none
 Version:        06.10.2010
 Description:    SD-cartd logging

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#if SDCARD_LOGFILE

#include <stdio.h>
#include <avr/pgmspace.h>
#include <stdbool.h>

#include "sdcard/fat16.h"
#include "sdcard/sdcard.h"
#include "sdcard/sd_logging.h"

#include "usart.h"
#include "timer.h"

#if USE_NTP
	#include "ntp.h"
#endif

#if USE_OW
	#include "1-wire/messung.h"
#endif

//#define SDLOG_DEBUG   usart_write
#define SDLOG_DEBUG(...)

void log_init()
{

	ntp_struct dt;
	decode_time(time, &dt);
	TM_DD = dt.day;
	TM_MM = dt.month;
	TM_YY = dt.year;

	logStatus.logTag	= TM_DD;
	logStatus.logMonat	= TM_MM;
	logStatus.logJahr	= TM_YY;

	sprintf_P((char *)logStatus.log_datei,PSTR("%04i%02i%02i.csv"),logStatus.logJahr,logStatus.logMonat,logStatus.logTag);
	SDLOG_DEBUG("\r\nlog_init: %s",logStatus.log_datei);

	if (logStatus.logfile) {
		f16_close(logStatus.logfile);
		logStatus.logfile = 0;
	}
	if (cwdir_ptr) {		// falls SD-Karte vorhanden
		logStatus.logfile = f16_open((char *)logStatus.log_datei,"a");
		// f16_fputs_P(PSTR("\r\n"),logStatus.logfile);		// verursacht Leerezeile
		f16_flush();		// Cache leeren
	}
	return;
}

void log_status(void)
{

	TM_hh = (time/3600)%24;
	TM_mm = (time/60)%60;
	TM_ss = time %60;

	// Logdatei fr Eingnge und Temparaturen
	if (logStatus.logfile) {
		f16_printf_P(logStatus.logfile,PSTR("%02i.%02i.%i;%02i:%02i:%02i;"),
		logStatus.logTag,logStatus.logMonat,logStatus.logJahr, TM_hh, TM_mm, TM_ss);

		// jetzt die Digital- und Analogwerte PORTA
		for(uint8_t i=0;i<8;i++)
		{
			f16_printf_P(logStatus.logfile,PSTR("%4i;"),(var_array[i]));
		}
		
		// und die 1-wire Sensoren falls vorhanden
		#if USE_OW
		for(uint8_t i=0;i<OW_MAXSENSORS;i++)
		{
			int16_t TWert = ow_array[i];
			// Vorzeichen
			if ( ow_array[i] < 0 ) {
				f16_printf_P(logStatus.logfile,PSTR("-"));
				TWert *= (-1);
			}
			f16_printf_P(logStatus.logfile,PSTR("%i,%01i;"),(TWert/10),(TWert%10));
		}
		//f16_printf_P(logStatus.logfile,PSTR("%4i,%2i;%4i;%4i;"),(ow_array[0]/10),(ow_array[1]/10),(ow_array[2]/10),(ow_array[3]/10));
		#endif
		
		f16_fputs_P(PSTR("\r\n"),logStatus.logfile);
		f16_flush();    // Cache leeren
	}
}


#endif	// SDCARD_LOGFILE