/*----------------------------------------------------------------------------
 Copyright:      Jens Mundhenke
 Author:         Jens Mundhenke
 Remarks:		 Used on the IP-stack of Ulrich Radig
  known Problems:
 Version:        16.03.2009
 Description:    Sending syslog-Messages

 This module sends a string to a syslog server.

 The IP of the server has to be stored in syslog_server_ip (similar to other
 IP addresses used by the stack). 

 Weitere Informationen:
 http://www.faqs.org/rfcs/rfc3164.html
 http://www.monitorware.com/common/en/articles/syslog-described.php

--------------------------------------------------------------------------------
 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation verffentlicht,
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
 (nach Ihrer Option) jeder spteren Version.

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben Sie an die Free Software Foundation,
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
------------------------------------------------------------------------------*/

#include "config.h"

#if USE_SYSLOG

#include <stdlib.h>
#include <string.h>
#include "stack.h"
#include "usart.h"
#include "syslog.h"

//#define SYSLOG_DEBUG	usart_write
#define SYSLOG_DEBUG(...)
//#define SYSLOG_ERROR	usart_write
#define SYSLOG_ERROR(...)

//-----------------------------------------------------------------------------------
// syslog_send - sending msg to syslog server defined in EEPROM
//				 priority and facility can be set according to RFC, see syslog.h
//				 for constants
void syslog_send ( unsigned int prio, unsigned int facility, char *msg )
{
	char *packet = (char *)(eth_buffer+UDP_DATA_START);	// pointer to begin of buffer
	char *pp = packet;										// write pointer to buffer
	int i;

	SYSLOG_DEBUG("SYSLOG send %s\r\n", msg);

	//send ARP Request to get MAC of SYSLOG server
	unsigned long tmp_ip = (*(unsigned long*)&syslog_server_ip[0]);
	if ( syslog_server_ip[3] != 0xFF )				// quickhack for broadcast address, only valid for C-class net
		if ( arp_entry_search(tmp_ip) >= MAX_ARP_ENTRY ) //Arp-Request senden?
			for ( i=0; i<SYSLOG_ARPRETRIES && (arp_request(tmp_ip) != 1); i++ );

	// continue even if the server has not replied to ARP request, packet will be sent
	// as broadcast
	if (i>=SYSLOG_ARPRETRIES) {
	  SYSLOG_ERROR("SYSLOG server not found!\r\n");
	}

	// building the packet structure
	*pp++ = '<';								// see RFC for simple header <..>
	itoa ( prio + (facility << 3), pp, 10 );	// copy fac/prio to buffer
	pp += strlen(pp);							// advance to end of written data
	*pp++ = '>';								// add the terminating >
	strcpy ( pp, msg );							// copy message to buffer

	// send SYSLOG packet to server (or broadcast it)
	create_new_udp_packet(strlen(packet),SYSLOG_CLIENT_PORT,SYSLOG_SERVER_PORT,tmp_ip);
}

#endif