/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        24.10.2007
 Description:    Timer Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "usart.h"
#include "stack.h"
#include "cmd.h"

#if USE_DNS
	#include "dnsc.h"
#endif

#if USE_NTP
	#include "ntp.h"
#endif

#if USE_DHCP
	#include "dhcpc.h"
#endif

#if (UDPSEND_SPORT!=0)
	#include "udp_send/udp_send.h"
#endif

#if SDCARD_COMMANDS
	#include "sdcard/fat16.h"
#endif

#if SDCARD_LOGFILE
	#include <stdbool.h>
	#include "sdcard/fat16.h"
	#include "sdcard/sd_logging.h"
#endif

#include "timer.h"

//----------------------------------------------------------------------------
//Diese Routine startet und inizialisiert den Timer
void timer_init (void)
{
	TCCR1B |= (1<<WGM12) | (1<<CS10 | 0<<CS11 | 1<<CS12);
	TCNT1 = 0;
	OCR1A = (F_CPU / 1024) - 1;
	TIMSK1 |= (1 << OCIE1A);
	return;
};

//----------------------------------------------------------------------------
//Timer Interrupt

ISR (TIMER1_COMPA_vect)
{

	time++;		// Sekunde hochzhlen

    if((stack_watchdog++) > WTT)
    {
        RESET();
	}
    eth.timer = 1;
	
	#if USE_DNS
	if ( dns_timer > 0 ) dns_timer--;		// dnsc interner Timer
	#endif //USE_DNS

	#if USE_NTP
	if ( ntp_resync > 0 ) ntp_resync--;
	if ( ntp_timer > 0 ) ntp_timer--;		// ntp interner Timer
	#endif //USE_NTP
	
	#if USE_DHCP
	if ( dhcp_lease > 0 ) dhcp_lease--;
    if ( dhcp_timer > 0 ) dhcp_timer--;	// dhcpc interner Timer
    #endif //USE_DHCP
	
	#if (LCD_BACKLIGHT && LCD_BL_AUTO)
	if ( lcd_bl_timer > 0 ) lcd_bl_timer--;
	#endif

	#if (UDPSEND_SPORT!=0)
	udp_send_timer--;
	#endif

	#if SDCARD_LOGFILE
	if( sdcard_log_timer > 0 ) sdcard_log_timer--;
	#endif

}

#if FAT16_DATETIME_SUPPORT
void get_datetime(uint16_t* year, uint8_t* month, uint8_t* day, uint8_t* hour, uint8_t* min, uint8_t* sec)
{
        *year   = TM_YY + 2000;
        *month  = TM_MM;
        *day    = TM_DD;

        *hour   = TM_hh;
        *min    = TM_mm;
        *sec    = TM_ss;
}
#endif
